@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22763 $
- * $Author: csoutheren $
- * $Date: 2009-05-28 22:54:55 -0500 (Thu, 28 May 2009) $
+ * $Revision: 23711 $
+ * $Author: rjongbloed $
+ * $Date: 2009-10-28 03:45:13 -0500 (Wed, 28 Oct 2009) $
  */
 
 #ifndef PTLIB_PXMLRPC_H
@@ -80,7 +80,7 @@ class PXMLRPC : public PObject
 
     PXMLRPC(
       const PURL & url,
-      unsigned options = 0
+      PXML::Options options = PXML::NoOptions
     );
 
     void SetTimeout(const PTimeInterval & t) { timeout = t; }
@@ -103,7 +103,7 @@ class PXMLRPC : public PObject
     PINDEX        faultCode;
     PString       faultText;
     PTimeInterval timeout;
-    unsigned      options;
+    PXML::Options m_options;
 };
 
 /////////////////////////////////////////////////////////////////
