@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25892 $
+ * $Revision: 26933 $
  * $Author: rjongbloed $
- * $Date: 2011-05-25 20:27:19 -0500 (Wed, 25 May 2011) $
+ * $Date: 2012-02-02 21:17:20 -0600 (Thu, 02 Feb 2012) $
  */
 
 #ifndef PTLIB_VXML_H
@@ -64,25 +64,32 @@ class PVXMLGrammar : public PObject
 {
   PCLASSINFO(PVXMLGrammar, PObject);
   public:
-    PVXMLGrammar(PXMLElement * field);
-    virtual PBoolean OnUserInput(const char /*ch*/) { return true; }
-    virtual void Stop() { }
+    PVXMLGrammar(PVXMLSession & session, PXMLElement & field);
 
-    PString GetValue() const { return value; }
-    PXMLElement * GetField() { return field; }
+    virtual void OnUserInput(const char ch) = 0;
+    virtual void Start();
+    virtual bool Process();
 
-    enum GrammarState { 
-      FILLED,       ///< got something that matched the grammar
-      NOINPUT,      ///< timeout or still waiting to match
-      NOMATCH,      ///< recognized something but didn't match the grammar
-      HELP };       ///< help keyword
+    enum GrammarState {
+      Idle,         ///< Not yet started
+      Started,      ///< Grammar awaiting input
+      Filled,       ///< got something that matched the grammar
+      NoInput,      ///< timeout or still waiting to match
+      NoMatch,      ///< recognized something but didn't match the grammar
+      Help          ///< help keyword
+    };
 
-    GrammarState GetState() const { return state; }
+    GrammarState GetState() const { return m_state; }
 
   protected:
-    PXMLElement * field;
-    PString value;
-    GrammarState state;
+    PDECLARE_NOTIFIER(PTimer, PVXMLGrammar, OnTimeout);
+
+    PVXMLSession & m_session;
+    PXMLElement  & m_field;
+    PString        m_value;
+    GrammarState   m_state;
+    PTimer         m_timer;
+    PMutex         m_mutex;
 };
 
 
@@ -92,7 +99,9 @@ class PVXMLMenuGrammar : public PVXMLGrammar
 {
   PCLASSINFO(PVXMLMenuGrammar, PVXMLGrammar);
   public:
-    PVXMLMenuGrammar(PXMLElement * field);
+    PVXMLMenuGrammar(PVXMLSession & session, PXMLElement & field);
+    virtual void OnUserInput(const char ch);
+    virtual bool Process();
 };
 
 
@@ -102,14 +111,20 @@ class PVXMLDigitsGrammar : public PVXMLGrammar
 {
   PCLASSINFO(PVXMLDigitsGrammar, PVXMLGrammar);
   public:
-    PVXMLDigitsGrammar(PXMLElement * field, PINDEX minDigits, PINDEX maxDigits, PString terminators);
-    PBoolean OnUserInput(const char ch);
-    virtual void Stop();
+    PVXMLDigitsGrammar(
+      PVXMLSession & session,
+      PXMLElement & field,
+      PINDEX minDigits,
+      PINDEX maxDigits,
+      PString terminators
+    );
+
+    virtual void OnUserInput(const char ch);
 
   protected:
-    PINDEX minDigits;
-    PINDEX maxDigits;
-    PString terminators;
+    PINDEX  m_minDigits;
+    PINDEX  m_maxDigits;
+    PString m_terminators;
 };
 
 
@@ -150,17 +165,7 @@ class PVXMLCache : public PMutex
 
 class PVXMLChannel;
 
-class PVXMLChannelInterface {
-  public:
-    virtual ~PVXMLChannelInterface() { }
-    virtual void RecordEnd() = 0;
-    virtual void OnEndRecording(const PString & channelName) = 0;
-    virtual void Trigger() = 0;
-};
-
-//////////////////////////////////////////////////////////////////
-
-class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
+class PVXMLSession : public PIndirectChannel
 {
   PCLASSINFO(PVXMLSession, PIndirectChannel);
   public:
@@ -170,13 +175,13 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
     // new functions
     PTextToSpeech * SetTextToSpeech(PTextToSpeech * tts, PBoolean autoDelete = false);
     PTextToSpeech * SetTextToSpeech(const PString & ttsName);
-    PTextToSpeech * GetTextToSpeech() const { return textToSpeech; }
+    PTextToSpeech * GetTextToSpeech() const { return m_textToSpeech; }
 
     virtual PBoolean Load(const PString & source);
-    virtual PBoolean LoadFile(const PFilePath & file);
+    virtual PBoolean LoadFile(const PFilePath & file, const PString & firstForm = PString::Empty());
     virtual PBoolean LoadURL(const PURL & url);
-    virtual PBoolean LoadVXML(const PString & xml);
-    virtual PBoolean IsLoaded() const { return loaded; }
+    virtual PBoolean LoadVXML(const PString & xml, const PString & firstForm = PString::Empty());
+    virtual PBoolean IsLoaded() const { return m_xml.IsLoaded(); }
 
     virtual PBoolean Open(const PString & mediaFormat);
     virtual PBoolean Close();
@@ -184,8 +189,8 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
     virtual PBoolean Execute();
 
     PVXMLChannel * GetAndLockVXMLChannel();
-    void UnLockVXMLChannel() { sessionMutex.Signal(); }
-    PMutex & GetSessionMutex() { return sessionMutex; }
+    void UnLockVXMLChannel() { m_sessionMutex.Signal(); }
+    PMutex & GetSessionMutex() { return m_sessionMutex; }
 
     virtual PBoolean LoadGrammar(PVXMLGrammar * grammar);
 
@@ -197,6 +202,7 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
     virtual PBoolean PlayCommand(const PString & data, PINDEX repeat = 1, PINDEX delay = 0);
     virtual PBoolean PlayResource(const PURL & url, PINDEX repeat = 1, PINDEX delay = 0);
     virtual PBoolean PlayTone(const PString & toneSpec, PINDEX repeat = 1, PINDEX delay = 0);
+    virtual PBoolean PlayElement(PXMLElement & element);
 
     //virtual PBoolean PlayMedia(const PURL & url, PINDEX repeat = 1, PINDEX delay = 0);
     virtual PBoolean PlaySilence(PINDEX msecs = 0);
@@ -209,120 +215,122 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
 
     virtual PBoolean StartRecording(const PFilePath & fn, PBoolean recordDTMFTerm, const PTimeInterval & recordMaxTime, const PTimeInterval & recordFinalSilence);
     virtual PBoolean EndRecording();
-    virtual PBoolean IsPlaying() const;
-    virtual PBoolean IsRecording() const;
 
-    virtual PBoolean OnUserInput(const PString & str);
+    virtual void OnUserInput(const PString & str);
 
     PString GetXMLError() const;
 
     virtual void OnEndDialog();
-    virtual void OnEndSession()         { }
-    virtual void OnTransfer(const PString & /*destination*/, bool /*bridged*/) { }
+    virtual void OnEndSession();
+
+    enum TransferType {
+      BridgedTransfer,
+      BlindTransfer,
+      ConsultationTransfer
+    };
+    virtual bool OnTransfer(const PString & /*destination*/, TransferType /*type*/) { return false; }
+    void SetTransferComplete(bool state);
 
     const PStringToString & GetVariables() { return m_variables; }
     virtual PCaselessString GetVar(const PString & str) const;
     virtual void SetVar(const PString & ostr, const PString & val);
     virtual PString EvaluateExpr(const PString & oexpr);
 
+    static PTimeInterval StringToTime(const PString & str);
+
     virtual PBoolean RetreiveResource(const PURL & url, PString & contentType, PFilePath & fn, PBoolean useCache = true);
 
     PDECLARE_NOTIFIER(PThread, PVXMLSession, VXMLExecute);
 
-    void SetCallingToken( PString& token ) { callingCallToken = token; }
-
-    PXMLElement * FindHandler(const PString & event);
+    bool SetCurrentForm(const PString & id, bool fullURI);
+    bool GoToEventHandler(PXMLElement & element, const PString & eventName);
 
     // overrides from VXMLChannelInterface
-    virtual void OnEndRecording(const PString & channelName);
-    virtual void RecordEnd();
+    virtual void OnEndRecording();
     virtual void Trigger();
 
 
-    virtual PBoolean TraverseAudio();
-    virtual PBoolean TraverseBreak();
-    virtual PBoolean TraverseValue();
-    virtual PBoolean TraverseSayAs();
-    virtual PBoolean TraverseGoto();
-    virtual PBoolean TraverseGrammar();
-    virtual PBoolean TraverseRecord();
-    virtual PBoolean TraverseIf();
-    virtual PBoolean TraverseExit();
-    virtual PBoolean TraverseVar();
-    virtual PBoolean TraverseSubmit();
-    virtual PBoolean TraverseMenu();
-    virtual PBoolean TraverseChoice();
-    virtual PBoolean TraverseProperty();
-    virtual PBoolean TraverseDisconnect();
-    virtual PBoolean TraverseForm();
-    virtual PBoolean TraversePrompt();
-    virtual PBoolean TraverseField();
-    virtual PBoolean TraversedField();
-    virtual PBoolean TraversedTransfer();
+    virtual PBoolean TraverseAudio(PXMLElement & element);
+    virtual PBoolean TraverseBreak(PXMLElement & element);
+    virtual PBoolean TraverseValue(PXMLElement & element);
+    virtual PBoolean TraverseSayAs(PXMLElement & element);
+    virtual PBoolean TraverseGoto(PXMLElement & element);
+    virtual PBoolean TraverseGrammar(PXMLElement & element);
+    virtual PBoolean TraverseRecord(PXMLElement & element);
+    virtual PBoolean TraversedRecord(PXMLElement & element);
+    virtual PBoolean TraverseIf(PXMLElement & element);
+    virtual PBoolean TraverseExit(PXMLElement & element);
+    virtual PBoolean TraverseVar(PXMLElement & element);
+    virtual PBoolean TraverseSubmit(PXMLElement & element);
+    virtual PBoolean TraverseMenu(PXMLElement & element);
+    virtual PBoolean TraversedMenu(PXMLElement & element);
+    virtual PBoolean TraverseChoice(PXMLElement & element);
+    virtual PBoolean TraverseProperty(PXMLElement & element);
+    virtual PBoolean TraverseDisconnect(PXMLElement & element);
+    virtual PBoolean TraverseForm(PXMLElement & element);
+    virtual PBoolean TraversedForm(PXMLElement & element);
+    virtual PBoolean TraversePrompt(PXMLElement & element);
+    virtual PBoolean TraverseField(PXMLElement & element);
+    virtual PBoolean TraversedField(PXMLElement & element);
+    virtual PBoolean TraverseFilled(PXMLElement & element);
+    virtual PBoolean TraverseNoInput(PXMLElement & element);
+    virtual PBoolean TraverseNoMatch(PXMLElement & element);
+    virtual PBoolean TraverseError(PXMLElement & element);
+    virtual PBoolean TraverseCatch(PXMLElement & element);
+    virtual PBoolean TraverseTransfer(PXMLElement & element);
+    virtual PBoolean TraversedTransfer(PXMLElement & element);
+
+    __inline PVXMLChannel * GetVXMLChannel() const { return (PVXMLChannel *)readChannel; }
 
   protected:
-    void Initialise();
-
-    virtual void ProcessUserInput();
-    virtual void ProcessGrammar();
-    virtual void ProcessNode();
-    virtual bool NextNode();
+    virtual bool ProcessNode();
+    virtual bool ProcessEvents();
+    virtual bool ProcessGrammar();
+    virtual bool NextNode(bool skipChildren);
 
     void SayAs(const PString & className, const PString & text);
     void SayAs(const PString & className, const PString & text, const PString & voice);
 
-    static PTimeInterval StringToTime(const PString & str);
-
     PURL NormaliseResourceName(const PString & src);
 
-    PXMLElement * FindForm(const PString & id);
+    PMutex           m_sessionMutex;
 
-    PSyncPoint waitForEvent;
+    PURL             m_rootURL;
+    PXML             m_xml;
 
-    PMutex sessionMutex;
+    PTextToSpeech *  m_textToSpeech;
+    bool             m_autoDeleteTextToSpeech;
 
-    PXML xmlFile;
+    PThread     *    m_vxmlThread;
+    bool             m_abortVXML;
+    PSyncPoint       m_waitForEvent;
+    PXMLObject  *    m_currentNode;
+    bool             m_xmlChanged;
+    bool             m_speakNodeData;
 
-    PVXMLGrammar * activeGrammar;
-    PBoolean listening;                 // true if waiting for recognition events
-    int timeout;                    // timeout in msecs for the current recognition
+    PVXMLGrammar *   m_grammar;
+    char             m_defaultMenuDTMF;
 
-    PStringToString m_variables;
+    PStringToString  m_variables;
+    PString          m_variableScope;
 
-    PMutex userInputMutex;
-    std::queue<char> userInputQueue;
+    std::queue<char> m_userInputQueue;
+    PMutex           m_userInputMutex;
 
-    PBoolean recording;
-    PFilePath recordFn;
-    PBoolean recordDTMFTerm;
-    PTimeInterval recordMaxTime;
-    PTimeInterval recordFinalSilence;
-    PSyncPoint    recordSync;
+    enum {
+      NotRecording,
+      RecordingInProgress,
+      RecordingComplete
+    }    m_recordingStatus;
+    bool m_recordStopOnDTMF;
 
-    PBoolean loaded;
-    PURL rootURL;
-    PBoolean emptyAction;
-
-    PThread * vxmlThread;
-    bool m_abortVXML;
-
-    PString mediaFormat;
-    PVXMLChannel * & vxmlChannel;
-
-    PTextToSpeech * textToSpeech;
-    PBoolean autoDeleteTextToSpeech;
-
-    PXMLElement * currentForm;
-    PXMLElement * currentField;
-    PXMLObject  * currentNode;
-    bool          m_speakNodeData;
-
-  private:
-    PString       callingCallToken;
-    PSyncPoint    answerSync;
-    PString       grammarResult;
-    PString       eventName;
-    PINDEX        defaultDTMF;
+    enum {
+      NotTransfering,
+      TransferInProgress,
+      TransferFailed,
+      TransferSuccessful
+    }     m_transferStatus;
+    PTime m_transferStartTime;
 };
 
 
@@ -332,37 +340,46 @@ class PVXMLRecordable : public PObject
 {
   PCLASSINFO(PVXMLRecordable, PObject);
   public:
-    PVXMLRecordable()
-    { consecutiveSilence = 0; finalSilence = 3000; maxDuration = 30000; }
+    PVXMLRecordable();
 
     virtual PBoolean Open(const PString & arg) = 0;
 
-    virtual void Record(PVXMLChannel & incomingChannel) = 0;
-
-    virtual void OnStart() { }
+    virtual bool OnStart(PVXMLChannel & incomingChannel) = 0;
+    virtual void OnStop() { }
 
     virtual PBoolean OnFrame(PBoolean /*isSilence*/) { return false; }
 
-    virtual void OnStop() { }
-
     void SetFinalSilence(unsigned v)
-    { finalSilence = v; }
+    { m_finalSilence = v > 0 ? v : 60000; }
 
     unsigned GetFinalSilence()
-    { return finalSilence; }
+    { return m_finalSilence; }
 
     void SetMaxDuration(unsigned v)
-    { maxDuration = v; }
+    { m_maxDuration = v > 0 ? v : 86400000; }
 
     unsigned GetMaxDuration()
-    { return maxDuration; }
+    { return m_maxDuration; }
 
   protected:
-    PTime silenceStart;
-    PTime recordStart;
-    unsigned finalSilence;
-    unsigned maxDuration;
-    unsigned consecutiveSilence;
+    PSimpleTimer m_silenceTimer;
+    PSimpleTimer m_recordTimer;
+    unsigned     m_finalSilence;
+    unsigned     m_maxDuration;
+};
+
+//////////////////////////////////////////////////////////////////
+
+class PVXMLRecordableFilename : public PVXMLRecordable
+{
+  PCLASSINFO(PVXMLRecordableFilename, PVXMLRecordable);
+  public:
+    PBoolean Open(const PString & arg);
+    bool OnStart(PVXMLChannel & incomingChannel);
+    PBoolean OnFrame(PBoolean isSilence);
+
+  protected:
+    PFilePath m_fileName;
 };
 
 //////////////////////////////////////////////////////////////////
@@ -371,52 +388,41 @@ class PVXMLPlayable : public PObject
 {
   PCLASSINFO(PVXMLPlayable, PObject);
   public:
-    PVXMLPlayable()
-    { repeat = 1; delay = 0; sampleFrequency = 8000; autoDelete = false; delayDone = false; }
+    PVXMLPlayable();
 
-    virtual PBoolean Open(PVXMLChannel & /*chan*/, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
+    virtual PBoolean Open(PVXMLChannel & chan, const PString & arg, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
 
-    virtual PBoolean Open(PVXMLChannel & chan, const PString & arg, PINDEX delay, PINDEX repeat, PBoolean v);
-
-    virtual void Play(PVXMLChannel & outgoingChannel) = 0;
-
-    virtual void OnRepeat(PVXMLChannel & /*outgoingChannel*/)
-    { }
-
-    virtual void OnStart() { }
-
-    virtual void OnStop() { }
+    virtual bool OnStart() = 0;
+    virtual bool OnRepeat();
+    virtual bool OnDelay();
+    virtual void OnStop();
 
     virtual void SetRepeat(PINDEX v) 
-    { repeat = v; }
+    { m_repeat = v; }
 
     virtual PINDEX GetRepeat() const
-    { return repeat; }
+    { return m_repeat; }
 
     virtual PINDEX GetDelay() const
-    { return delay; }
+    { return m_delay; }
 
     void SetFormat(const PString & fmt)
-    { format = fmt; }
+    { m_format = fmt; }
 
     void SetSampleFrequency(unsigned rate)
-    { sampleFrequency = rate; }
-
-    virtual PBoolean ReadFrame(PVXMLChannel & channel, void * buf, PINDEX len);
-
-    virtual PBoolean Rewind(PChannel *) 
-    { return false; }
+    { m_sampleFrequency = rate; }
 
     friend class PVXMLChannel;
 
   protected:
-    PString arg;
-    PINDEX repeat;
-    PINDEX delay;
-    PString format;
-    unsigned sampleFrequency;
-    PBoolean autoDelete;
-    PBoolean delayDone; // very tacky flag used to indicate when the post-play delay has been done
+    PVXMLChannel * m_vxmlChannel;
+    PChannel * m_subChannel;
+    PINDEX   m_repeat;
+    PINDEX   m_delay;
+    PString  m_format;
+    unsigned m_sampleFrequency;
+    bool     m_autoDelete;
+    bool     m_delayDone; // very tacky flag used to indicate when the post-play delay has been done
 };
 
 //////////////////////////////////////////////////////////////////
@@ -425,8 +431,7 @@ class PVXMLPlayableStop : public PVXMLPlayable
 {
   PCLASSINFO(PVXMLPlayableStop, PVXMLPlayable);
   public:
-    virtual void Play(PVXMLChannel & outgoingChannel);
-    virtual PBoolean ReadFrame(PVXMLChannel & channel, void *, PINDEX);
+    virtual bool OnStart();
 };
 
 //////////////////////////////////////////////////////////////////
@@ -435,10 +440,10 @@ class PVXMLPlayableURL : public PVXMLPlayable
 {
   PCLASSINFO(PVXMLPlayableURL, PVXMLPlayable);
   public:
-    PBoolean Open(PVXMLChannel & chan, const PString & url, PINDEX delay, PINDEX repeat, PBoolean v);
-    void Play(PVXMLChannel & outgoingChannel);
+    virtual PBoolean Open(PVXMLChannel & chan, const PString & arg, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
+    virtual bool OnStart();
   protected:
-    PURL url;
+    PURL m_url;
 };
 
 //////////////////////////////////////////////////////////////////
@@ -447,12 +452,12 @@ class PVXMLPlayableData : public PVXMLPlayable
 {
   PCLASSINFO(PVXMLPlayableData, PVXMLPlayable);
   public:
-    PBoolean Open(PVXMLChannel & chan, const PString & fn, PINDEX delay, PINDEX repeat, PBoolean v);
+    virtual PBoolean Open(PVXMLChannel & chan, const PString & arg, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
     void SetData(const PBYTEArray & data);
-    void Play(PVXMLChannel & outgoingChannel);
-    PBoolean Rewind(PChannel * chan);
+    virtual bool OnStart();
+    virtual bool OnRepeat();
   protected:
-    PBYTEArray data;
+    PBYTEArray m_data;
 };
 
 //////////////////////////////////////////////////////////////////
@@ -463,9 +468,9 @@ class PVXMLPlayableTone : public PVXMLPlayableData
 {
   PCLASSINFO(PVXMLPlayableTone, PVXMLPlayableData);
   public:
-    PBoolean Open(PVXMLChannel & chan, const PString & toneSpec, PINDEX delay, PINDEX repeat, PBoolean v);
+    virtual PBoolean Open(PVXMLChannel & chan, const PString & arg, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
   protected:
-    PTones tones;
+    PTones m_tones;
 };
 
 //////////////////////////////////////////////////////////////////
@@ -474,56 +479,43 @@ class PVXMLPlayableCommand : public PVXMLPlayable
 {
   PCLASSINFO(PVXMLPlayableCommand, PVXMLPlayable);
   public:
-    PVXMLPlayableCommand();
-    void Play(PVXMLChannel & outgoingChannel);
-    void OnStop();
+    virtual PBoolean Open(PVXMLChannel & chan, const PString & arg, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
+    virtual bool OnStart();
+    virtual void OnStop();
 
   protected:
-    PPipeChannel * pipeCmd;
+    PString m_command;
 };
 
 //////////////////////////////////////////////////////////////////
 
-class PVXMLPlayableFilename : public PVXMLPlayable
+class PVXMLPlayableFile : public PVXMLPlayable
 {
-  PCLASSINFO(PVXMLPlayableFilename, PVXMLPlayable);
+  PCLASSINFO(PVXMLPlayableFile, PVXMLPlayable);
   public:
-    PBoolean Open(PVXMLChannel & chan, const PString & fn, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
-    void Play(PVXMLChannel & outgoingChannel);
-    void OnStop();
-    virtual PBoolean Rewind(PChannel * chan);
+    virtual PBoolean Open(PVXMLChannel & chan, const PString & arg, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
+    virtual bool OnStart();
+    virtual bool OnRepeat();
+    virtual void OnStop();
   protected:
-    PFilePath fn;
+    PFilePath m_filePath;
 };
 
 //////////////////////////////////////////////////////////////////
 
-class PVXMLPlayableFilenameList : public PVXMLPlayable
+class PVXMLPlayableFileList : public PVXMLPlayableFile
 {
-  PCLASSINFO(PVXMLPlayableFilenameList, PVXMLPlayable);
+  PCLASSINFO(PVXMLPlayableFileList, PVXMLPlayableFile);
   public:
-    PBoolean Open(PVXMLChannel & chan, const PStringArray & filenames, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
-    void Play(PVXMLChannel & outgoingChannel)
-    { OnRepeat(outgoingChannel); }
-    void OnRepeat(PVXMLChannel & outgoingChannel);
-    void OnStop();
+    PVXMLPlayableFileList();
+    virtual PBoolean Open(PVXMLChannel & chan, const PString & arg, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
+    virtual PBoolean Open(PVXMLChannel & chan, const PStringArray & filenames, PINDEX delay, PINDEX repeat, PBoolean autoDelete);
+    virtual bool OnStart();
+    virtual bool OnRepeat();
+    virtual void OnStop();
   protected:
-    PINDEX currentIndex;
-    PStringArray filenames;
-};
-
-//////////////////////////////////////////////////////////////////
-
-class PVXMLRecordableFilename : public PVXMLRecordable
-{
-  PCLASSINFO(PVXMLRecordableFilename, PVXMLRecordable);
-  public:
-    PBoolean Open(const PString & arg);
-    void Record(PVXMLChannel & incomingChannel);
-    PBoolean OnFrame(PBoolean isSilence);
-
-  protected:
-    PFilePath fn;
+    PStringArray m_fileNames;
+    PINDEX       m_currentIndex;
 };
 
 //////////////////////////////////////////////////////////////////
@@ -539,7 +531,7 @@ class PVXMLChannel : public PDelayChannel
     PVXMLChannel(unsigned frameDelay, PINDEX frameSize);
     ~PVXMLChannel();
 
-    virtual PBoolean Open(PVXMLChannelInterface * vxml);
+    virtual PBoolean Open(PVXMLSession * session);
 
     // overrides from PIndirectChannel
     virtual PBoolean IsOpen() const;
@@ -562,7 +554,7 @@ class PVXMLChannel : public PDelayChannel
 
     PBoolean StartRecording(const PFilePath & fn, unsigned finalSilence = 3000, unsigned maxDuration = 30000);
     PBoolean EndRecording();
-    PBoolean IsRecording() const { return recording; }
+    PBoolean IsRecording() const { return m_recordable != NULL; }
 
     // Outgoing channel functions
     virtual PBoolean ReadFrame(void * buffer, PINDEX amount) = 0;
@@ -582,45 +574,36 @@ class PVXMLChannel : public PDelayChannel
     { return QueuePlayable("Command", cmd, repeat, delay, true); }
 
     virtual void FlushQueue();
-    virtual PBoolean IsPlaying() const { return currentPlayItem != NULL || playQueue.GetSize() > 0; }
+    virtual PBoolean IsPlaying() const { return m_currentPlayItem != NULL || m_playQueue.GetSize() > 0; }
 
-    void SetPause(PBoolean pause) { paused = pause; }
+    void SetPause(PBoolean pause) { m_paused = pause; }
 
-    void SetName(const PString & name) { channelName = name; }
-
-    unsigned GetSampleFrequency() const { return sampleFrequency; }
+    unsigned GetSampleFrequency() const { return m_sampleFrequency; }
 
     void SetSilence(unsigned msecs);
 
   protected:
-    PVXMLChannelInterface * vxmlInterface;
+    PVXMLSession * m_vxmlSession;
 
-    unsigned sampleFrequency;
+    unsigned m_sampleFrequency;
     PString mediaFormat;
     PString wavFilePrefix;
 
-    PMutex channelWriteMutex;
-    PMutex channelReadMutex;
-    PBoolean closed;
+    PMutex   m_channelWriteMutex;
+    PMutex   m_channelReadMutex;
+    bool     m_closed;
+    bool     m_paused;
+    PINDEX   m_totalData;
 
     // Incoming audio variables
-    PBoolean recording;
-    PVXMLRecordable * recordable;
-    unsigned finalSilence;
-    unsigned silenceRun;
+    PVXMLRecordable * m_recordable;
+    unsigned          m_finalSilence;
+    unsigned          m_silenceRun;
 
     // Outgoing audio variables
-    PMutex queueMutex;
-    PVXMLQueue playQueue;
-    PVXMLPlayable * currentPlayItem;
-
-    PBoolean paused;
-    PTimer   m_silenceTimer;
-    int totalData;
-
-    // "channelname" (which is the name of the <record> tag) so
-    // results can be saved in vxml session variable
-    PString channelName;
+    PVXMLQueue      m_playQueue;
+    PVXMLPlayable * m_currentPlayItem;
+    PSimpleTimer    m_silenceTimer;
 };
 
 
@@ -630,8 +613,11 @@ class PVXMLNodeHandler : public PObject
 {
     PCLASSINFO(PVXMLNodeHandler, PObject);
   public:
-    virtual bool Start(PVXMLSession & /*session*/, const PXMLElement & /*node*/) const { return false; }
-    virtual bool Finish(PVXMLSession & /*session*/, const PXMLElement & /*node*/) const { return false; }
+    // Return true for process node, false to skip and move to next sibling
+    virtual bool Start(PVXMLSession & /*session*/, PXMLElement & /*node*/) const { return true; }
+
+    // Return true to move to next sibling, false to stay at this node.
+    virtual bool Finish(PVXMLSession & /*session*/, PXMLElement & /*node*/) const { return true; }
 };
 
 
