@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22082 $
+ * $Revision: 26758 $
  * $Author: rjongbloed $
- * $Date: 2009-02-16 23:31:44 -0600 (Mon, 16 Feb 2009) $
+ * $Date: 2011-12-06 18:52:22 -0600 (Tue, 06 Dec 2011) $
  */
 
 
@@ -38,7 +38,7 @@
   public:
     ~PProcess();
 
-    void SignalTimerChange();
+    bool SignalTimerChange();
     // Signal to the timer thread that a change was made.
 
     virtual PBoolean IsServiceProcess() const;
@@ -68,10 +68,7 @@
   friend UINT __stdcall PThread::MainFunction(void * thread);
   friend class PServiceProcess;
   friend class PApplication;
-#ifndef _WIN32_WCE
-  friend int PASCAL WinMain(HINSTANCE, HINSTANCE, LPSTR, int);
-#else
-  friend int PASCAL WinMain(HINSTANCE, HINSTANCE, LPTSTR, int);
-#endif
+
+  friend PDEFINE_WINMAIN(hInstance, hPrevInstance, lpCmdLine, nCmdShow);
 
 // End Of File ///////////////////////////////////////////////////////////////
