@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24524 $
+ * $Revision: 26758 $
  * $Author: rjongbloed $
- * $Date: 2010-06-21 20:05:24 -0500 (Mon, 21 Jun 2010) $
+ * $Date: 2011-12-06 18:52:22 -0600 (Tue, 06 Dec 2011) $
  */
 
 #ifndef PTLIB_PROCESS_H
@@ -67,11 +67,11 @@ extern "C" {\
 }
 #elif defined(_WIN32_WCE)
 #define PCREATE_PROCESS(cls) \
-  int WinMain(HINSTANCE hInst, HINSTANCE, LPWSTR cmdLine, int) \
+  PDEFINE_WINMAIN(hInstance, , lpCmdLine, ) \
     { \
       cls *pInstance = new cls(); \
-      pInstance->GetArguments().SetArgs(cmdLine); \
-      int terminationValue = pInstance->InternalMain(hInst); \
+      pInstance->GetArguments().SetArgs(lpCmdLine); \
+      int terminationValue = pInstance->InternalMain(hInstance); \
       delete pInstance; \
       return terminationValue; \
     }
