@@ -17,13 +17,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20725 $
- * $Author: ms30002000 $
- * $Date: 2008-08-22 18:21:16 +0000 (Fri, 22 Aug 2008) $
+ * $Revision: 21788 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
  */
 
-#ifndef _PQOS
-#define _PQOS
+#ifndef PTLIB_QOS_H
+#define PTLIB_QOS_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -104,4 +104,8 @@ public:
 
 };
 
-#endif // _PQOS
+
+#endif // PTLIB_QOS_H
+
+
+// End Of File ///////////////////////////////////////////////////////////////
