@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22598 $
+ * $Revision: 25554 $
  * $Author: rjongbloed $
- * $Date: 2009-05-13 04:17:54 +0000 (Wed, 13 May 2009) $
+ * $Date: 2011-04-13 03:34:50 -0500 (Wed, 13 Apr 2011) $
  */
  
 #ifndef PTLIB_SAFE_COLLECTION_H
@@ -92,7 +92,29 @@
   automatic unlocking of objects ones the pointer goes out of scope. It may
   also be used to lock each object of a collection in turn.
 
-  The enumeration
+  The enumeration of a PSafeCollection of PSafeObjects utilises the PSafePtr
+  class in a classic "for loop" manner.
+
+  <CODE>
+    for (PSafePtr<MyClass> iter(collection, PSafeReadWrite); iter != NULL; ++iter)
+      iter->Process();
+  </CODE>
+
+  There is one piece if important behaviour in the above. If while enumerating
+  a specic object in the collection, that object is "safely deleted", you are
+  guaranteed that the object is still usable and has not been phsyically
+  deleted, however it will no longer be in the collection, so the enumeration
+  will stop as it can no longer determine where in the collection it was.
+
+  What to do in this case is to take a "snapshot" at a point of time that can be safely and completely
+  iterated over:
+
+  <CODE>
+    PSafeList<MyClass> collCopy = collection;
+    for (PSafePtr<MyClass> iter(callCopy, PSafeReadWrite); iter != NULL; ++iter)
+      iter->Process();
+  </CODE>
+
  */
 class PSafeObject : public PObject
 {
@@ -114,7 +136,7 @@ class PSafeObject : public PObject
        deallocated) as the caller thread is using the object, but not
        necessarily at this time locking it.
 
-       If the function returns PFalse, then the object has been flagged for
+       If the function returns false, then the object has been flagged for
        deletion and the calling thread should immediately cease using the
        object.
 
@@ -136,7 +158,7 @@ class PSafeObject : public PObject
        It is recommended that the PSafePtr<> class is used to manage this
        rather than the application calling this function directly.
 
-       @return PTrue if reference count has reached zero and is not being
+       @return true if reference count has reached zero and is not being
                safely deleted elsewhere ie SafeRemove() not called
       */
     PBoolean SafeDereference();
@@ -148,7 +170,7 @@ class PSafeObject : public PObject
        occur and no read/write lock can be present for any read only locks to
        occur.
 
-       If the function returns PFalse, then the object has been flagged for
+       If the function returns false, then the object has been flagged for
        deletion and the calling thread should immediately cease use of the
        object, possibly executing the SafeDereference() function to remove
        any references it may have acquired.
@@ -179,7 +201,7 @@ class PSafeObject : public PObject
        occur and no read/write lock can be present for any read only locks to
        occur.
 
-       If the function returns PFalse, then the object has been flagged for
+       If the function returns false, then the object has been flagged for
        deletion and the calling thread should immediately cease use of the
        object, possibly executing the SafeDereference() function to remove
        any references it may have acquired.
@@ -348,7 +370,7 @@ class PSafeCollection : public PObject
     /**Remove all objects in collection.
       */
     virtual void RemoveAll(
-      PBoolean synchronous = PFalse  ///< Wait till objects are deleted before returning
+      PBoolean synchronous = false  ///< Wait till objects are deleted before returning
     );
 
     /**Disallow the automatic delete any objects that have been removed.
@@ -356,17 +378,17 @@ class PSafeCollection : public PObject
        deletion using PSafeObject::SafeRemove() and DeleteObject().
       */
     void AllowDeleteObjects(
-      PBoolean yes = PTrue   ///< New value for flag for deleting objects
+      PBoolean yes = true   ///< New value for flag for deleting objects
     ) { deleteObjects = yes; }
 
     /**Disallow the automatic delete any objects that have been removed.
        Objects are simply removed from the collection and not marked for
        deletion using PSafeObject::SafeRemove() and DeleteObject().
       */
-    void DisallowDeleteObjects() { deleteObjects = PFalse; }
+    void DisallowDeleteObjects() { deleteObjects = false; }
 
     /**Delete any objects that have been removed.
-       Returns PTrue if all objects in the collection have been removed and
+       Returns true if all objects in the collection have been removed and
        their pending deletions carried out.
       */
     virtual PBoolean DeleteObjectsToBeRemoved();
@@ -397,16 +419,22 @@ class PSafeCollection : public PObject
   //@}
 
   protected:
+    void CopySafeCollection(PCollection * other);
+    void CopySafeDictionary(PAbstractDictionary * other);
     void SafeRemoveObject(PSafeObject * obj);
     PDECLARE_NOTIFIER(PTimer, PSafeCollection, DeleteObjectsTimeout);
 
-    PCollection  *     collection;
+    PCollection      * collection;
     mutable PMutex     collectionMutex;
-    PBoolean               deleteObjects;
+    bool               deleteObjects;
     PList<PSafeObject> toBeRemoved;
     PMutex             removalMutex;
     PTimer             deleteObjectsTimer;
 
+  private:
+    PSafeCollection(const PSafeCollection & other) : PObject(other) { }
+    void operator=(const PSafeCollection &) { }
+
   friend class PSafePtrBase;
 };
 
@@ -506,7 +534,7 @@ class PSafePtrBase : public PObject
       */
     virtual void SetNULL();
 
-    /**Return PTrue if pointer is NULL.
+    /**Return true if pointer is NULL.
       */
     bool operator!() const { return currentObject == NULL; }
 
@@ -537,6 +565,7 @@ class PSafePtrBase : public PObject
   protected:
     virtual void Next();
     virtual void Previous();
+    virtual void DeleteObject(PSafeObject * obj);
 
     enum EnterSafetyModeOption {
       WithReference,
@@ -666,9 +695,14 @@ class PSafePtrMultiThreaded : public PSafePtrBase
   protected:
     virtual void Next();
     virtual void Previous();
+    virtual void DeleteObject(PSafeObject * obj);
+
+    void Lock() { m_mutex.Wait(); }
+    void Unlock();
 
   protected:
     mutable PMutex m_mutex;
+    PSafeObject  * m_objectToDelete;
 };
 
 
@@ -679,17 +713,17 @@ class PSafePtrMultiThreaded : public PSafePtrBase
   thread safe. Any thread can add, read, write or remove an object with both
   the object and the database of objects itself kept thread safe.
 
-  There are two modes of safe pointer, one that is enumerating a collection
+  There are two modes of safe pointer: one that is enumerating a collection;
   and one that is independent of the collection that the safe object is in.
-  There are some subtle semantics that must be observed in each of these two
+  There are subtle semantics that must be observed in each of these two
   modes especially when switching from one to the other.
 
   NOTE: the PSafePtr will allow safe and mutexed access to objects but may not
-  thread safe itself! This depends on the base calass being used. If the more
-  efficiant PSafePtrBase class is used you should not share PSafePtr instances
+  be thread safe itself! This depends on the base class being used. If the more
+  efficient PSafePtrBase class is used you should not share PSafePtr instances
   across threads.
 
-  See the PSafeObject class for more details. Especially in regard to
+  See the PSafeObject class for more details, especially in regards to
   enumeration of collections.
  */
 template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseClass
@@ -782,7 +816,7 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
      */
     PSafePtr & operator=(T * obj)
       {
-        Assign(obj);
+        this->Assign(obj);
         return *this;
       }
 
@@ -912,6 +946,30 @@ template <class Coll, class Base> class PSafeColl : public PSafeCollection
     PSafeColl()
       : PSafeCollection(new Coll)
       { }
+
+    /**Copy constructor for safe collection.
+       Note the left hand side will always have DisallowDeleteObjects() set.
+      */
+    PSafeColl(const PSafeColl & other)
+      : PSafeCollection(new Coll)
+    {
+      PWaitAndSignal lock2(other.collectionMutex);
+      CopySafeCollection(dynamic_cast<Coll *>(other.collection));
+    }
+
+    /**Assign one safe collection to another.
+       Note the left hand side will always have DisallowDeleteObjects() set.
+      */
+    PSafeColl & operator=(const PSafeColl & other)
+    {
+      if (&other != this) {
+        RemoveAll(true);
+        PWaitAndSignal lock1(collectionMutex);
+        PWaitAndSignal lock2(other.collectionMutex);
+        CopySafeCollection(dynamic_cast<Coll *>(other.collection));
+      }
+      return *this;
+    }
   //@}
 
   /**@name Operations */
@@ -922,7 +980,7 @@ template <class Coll, class Base> class PSafeColl : public PSafeCollection
       */
     virtual PSafePtr<Base> Append(
       Base * obj,       ///< Object to add to safe collection.
-      PSafetyMode mode = PSafeReference
+      PSafetyMode mode = PSafeReference   ///< Safety mode for returned locked PSafePtr
     ) {
         PWaitAndSignal mutex(collectionMutex);
         if (PAssert(collection->GetObjectsIndex(obj) == P_MAX_INDEX, "Cannot insert safe object twice") &&
@@ -1043,6 +1101,30 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
       */
     PSafeDictionaryBase()
       : PSafeCollection(new Coll) { }
+
+    /**Copy constructor for safe collection.
+       Note the left hand side will always have DisallowDeleteObjects() set.
+      */
+    PSafeDictionaryBase(const PSafeDictionaryBase & other)
+      : PSafeCollection(new Coll)
+    {
+      PWaitAndSignal lock2(other.collectionMutex);
+      CopySafeDictionary(dynamic_cast<Coll *>(other.collection));
+    }
+
+    /**Assign one safe collection to another.
+       Note the left hand side will always have DisallowDeleteObjects() set.
+      */
+    PSafeDictionaryBase & operator=(const PSafeDictionaryBase & other)
+    {
+      if (&other != this) {
+        RemoveAll(true);
+        PWaitAndSignal lock1(collectionMutex);
+        PWaitAndSignal lock2(other.collectionMutex);
+        CopySafeDictionary(dynamic_cast<Coll *>(other.collection));
+      }
+      return *this;
+    }
   //@}
 
   /**@name Operations */
@@ -1112,6 +1194,17 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
         ptr.SetSafetyMode(mode);
         return ptr;
       }
+
+    /**Get an array containing all the keys for the dictionary.
+      */
+    PArray<Key> GetKeys() const
+    {
+      PArray<Key> keys;
+      collectionMutex.Wait();
+      ((Coll *)collection)->AbstractGetKeys(keys);
+      collectionMutex.Signal();
+      return keys;
+    }
   //@}
 };
 
