@@ -26,14 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20565 $
+ * $Revision: 26804 $
  * $Author: rjongbloed $
- * $Date: 2008-07-11 11:19:15 +0200 (ven 11 jui 2008) $
+ * $Date: 2011-12-17 01:23:10 -0600 (Sat, 17 Dec 2011) $
  */
 
 
-#ifndef _PSOUND
-#define _PSOUND
+#ifndef PTLIB_SOUND_H
+#define PTLIB_SOUND_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -46,7 +46,7 @@
    entity that is abstracted for use here. Very little manipulation of the
    sounds are possible.
 
-   The most common sound to use is the static function #Beep()# which
+   The most common sound to use is the static function <code>Beep()</code> which
    emits the system standard "warning" or "attention" sound.
  */
 class PSound : public PBYTEArray
@@ -92,7 +92,7 @@ class PSound : public PBYTEArray
        format conversions between file and driver are performed.
 
        @return
-       PTrue if the sound is loaded successfully.
+       true if the sound is loaded successfully.
      */
     PBoolean Load(
       const PFilePath & filename   ///< Sound file to load.
@@ -102,7 +102,7 @@ class PSound : public PBYTEArray
        object.
 
        @return
-       PTrue if the sound is saved successfully.
+       true if the sound is saved successfully.
      */
     PBoolean Save(
       const PFilePath & filename   ///< Sound file to load.
@@ -152,17 +152,17 @@ class PSound : public PBYTEArray
 
   /**@name Miscellaneous functions */
   //@{
-    /**Play a sound file to the default device. If the #wait#
-       parameter is PTrue then the function does not return until the file has
-       been played. If PFalse then the sound play is begun asynchronously and
+    /**Play a sound file to the default device. If the <code>wait</code>
+       parameter is true then the function does not return until the file has
+       been played. If false then the sound play is begun asynchronously and
        the function returns immediately.
 
        @return
-       PTrue if the sound is playing or has played.
+       true if the sound is playing or has played.
      */
     static PBoolean PlayFile(
       const PFilePath & file, ///< Sound file to play.
-      PBoolean wait = PTrue        ///< Flag to play sound synchronously.
+      PBoolean wait = true        ///< Flag to play sound synchronously.
     );
 
     /// Play the "standard" warning beep for the platform.
@@ -186,16 +186,48 @@ class PSound : public PBYTEArray
 
 
 /**
-   This class is both an abstract class for a generalised sound channel, 
-   and an implementation of PSoundChannel for old code that is not plugin-aware.
-   When instantiated, it selects the first plugin of the base class "PSoundChannel"
+   Abstract class for a generalised sound channel, and an implementation of
+   PSoundChannel for old code that is not plugin-aware.
+   When instantiated, it selects the first plugin of the base class 
+   "PSoundChannel"
 
-   As an abstract class, this represents a sound schannel. Drivers for real, platform
-   dependent sound hardware will be ancestors of this class and can be found
-   in the plugins section of PWLib.
+   As an abstract class, this represents a sound channel. Drivers for real, 
+   platform dependent sound hardware will be ancestors of this class and 
+   can be found in the plugins section of PTLib.
 
-   A sound driver is either playing or recording. If simultaneous playing and
-   recording is desired, two instances of PSoundChannel must be created.
+   A sound channel is either playing or recording. If simultaneous
+   playing and recording is desired, two instances of PSoundChannel
+   must be created. It is an error for the same thread to attempt to
+   both read and write audio data to once instance of a PSoundChannel
+   class.
+
+   PSoundChannel instances are designed to be reentrant. The actual
+   usage model employed is left to the developer. One model could be
+   where one thread is responsible for construction, setup, opening and 
+   read/write operations. After creating and eventually opening the channel
+   this thread is responsible for handling read/writes fast enough to avoid
+   gaps in the generated audio stream.
+
+   Remaining operations may beinvoked from other threads.
+   This includes Close() and actually gathering the necessary data to
+   be sent to the device.
+
+   Besides the basic I/O task, the Read()/Write(() functions have well
+   defined timing characteristics. When a PSoundChannel instance is
+   used from Opal, the read/write operations are designed to also act
+   as timers so as to nicely space the generated network packets of
+   audio/ sound packets to the speaker.
+
+
+   Read and Writes of audio data to a PSoundChannel are blocking. The
+   length of time required to read/write a block of audio from/to a
+   PSoundChannel instance is equal to the time required for that block
+   of audio to record/play. So for a sound rate of 8khz, 240 samples,
+   it is going to take 30ms to do a read/write.
+
+   Since the Read()/Write(() functions have well defined
+   timing characteristics; they are designed to also act as timers in a loop
+   involving data transfers to/from the codecs.
 
    The sound is buffered and the size and number of buffers should be set
    before playing/recording. Each call to Write() will use one buffer, so care
@@ -205,7 +237,7 @@ class PSound : public PBYTEArray
    Similarly for reading, an entire buffer must be read before any of it is
    available to a Read() call. Note that once a buffer is filled you can read
    it a byte at a time if desired, but as soon as all the data in the buffer
-   is used returned, the next read will wait until the entire next buffer is
+   is used, the next read will wait until the entire next buffer is
    read from the hardware. So again, tailor the number and size of buffers to
    the application. To avoid being blocked until the buffer fills, you can use
    the StartRecording() function to initiate the buffer filling, and the
@@ -214,6 +246,7 @@ class PSound : public PBYTEArray
 
    Note that this sound channel is implicitly a linear PCM channel. No data
    conversion is performed on data to/from the channel.
+
  */
 class PSoundChannel : public PChannel
 {
@@ -223,6 +256,7 @@ class PSoundChannel : public PChannel
   /**@name Construction */
   //@{
     enum Directions {
+      Closed,
       Recorder,
       Player
     };
@@ -308,13 +342,9 @@ class PSoundChannel : public PChannel
 
     /**Get the name for the default sound devices/driver that is on this
        platform. Note that a named device may not necessarily do both
-       playing and recording so the arrays returned with the #dir#
+       playing and recording so the string returned with the <code>dir</code>
        parameter in each value is not necessarily the same.
 
-       This will return a list of uniqie device names across all of the available
-       drivers. If two drivers have identical names for devices, then the string
-       returned will be of the form driver+'\t'+device.
-
        @return
        A platform dependent string for the sound player/recorder.
      */
@@ -324,9 +354,13 @@ class PSoundChannel : public PChannel
 
     /**Get the list of all devices name for the default sound devices/driver that is on this
        platform. Note that a named device may not necessarily do both
-       playing and recording so the arrays returned with the #dir#
+       playing and recording so the arrays returned with the <code>dir</code>
        parameter in each value is not necessarily the same.
 
+       This will return a list of unique device names across all of the available
+       drivers. If two drivers have identical names for devices, then the string
+       returned will be of the form driver+'\\t'+device.
+
        @return
        Platform dependent strings for the sound player/recorder.
      */
@@ -339,7 +373,7 @@ class PSoundChannel : public PChannel
        platform specific and is as returned in the GetDevices() function.
 
        @return
-       PTrue if the sound device is valid for playing/recording.
+       true if the sound device is valid for playing/recording.
      */
     virtual PBoolean Open(
       const PString & device,       ///< Name of sound driver/device
@@ -352,13 +386,13 @@ class PSoundChannel : public PChannel
     /**Test if this instance of PSoundChannel is open.
 
        @return
-       PTrue if this instance is open.
+       true if this instance is open.
      */
     virtual PBoolean IsOpen() const;
 
-    /** Close the channel, shutting down the link to the data source.
+    /** Close the channel, shutting down the link to the data source. 
 
-       @return PTrue if the channel successfully closed.
+       @return true if the channel successfully closed.
      */
     virtual PBoolean Close();
 
@@ -372,10 +406,29 @@ class PSoundChannel : public PChannel
     /// Get the name of the open channel
     virtual PString GetName() const;
 
-    /**Abort the background playing/recording of the sound channel.
+    /// Get the direction of the channel
+    virtual Directions GetDirection() const
+    {
+      return activeDirection;
+    }
+
+    /// Get text representing the direction of the channel
+    static const char * GetDirectionText(Directions dir)
+    {
+      return (dir == Player)? "Playback" : ((dir == Recorder)? "Recording" : "Closed");
+    }
+
+    virtual const char * GetDirectionText() const
+    {
+      return GetDirectionText(activeDirection);
+    }
+
+    /** Abort the background playing/recording of the sound channel.
+        There will be a logic assertion if you attempt to Abort a
+        sound channel operation, when the device is currently closed.
 
        @return
-       PTrue if the sound has successfully been aborted.
+       true if the sound has successfully been aborted.
      */
     virtual PBoolean Abort();
   //@}
@@ -389,7 +442,7 @@ class PSoundChannel : public PChannel
        by information in the file being played.
 
        @return
-       PTrue if the format is valid.
+       true if the format is valid.
      */
     virtual PBoolean SetFormat(
       unsigned numChannels = 1,     ///< Number of channels eg mono/stereo
@@ -406,13 +459,13 @@ class PSoundChannel : public PChannel
     /// Get the sample size in bits per sample.
     virtual unsigned GetSampleSize() const;
 
-    /**Set the internal buffers for the sound channel I/O.
+    /**Set the internal buffers for the sound channel I/O. 
 
        Note that with Linux OSS, the size is always rounded up to the nearest
-       power of two, so 20000 => 32768.
+       power of two, so 20000 => 32768. 
 
        @return
-       PTrue if the sound device is valid for playing/recording.
+       true if the sound device is valid for playing/recording.
      */
     virtual PBoolean SetBuffers(
       PINDEX size,      ///< Size of each buffer
@@ -422,7 +475,7 @@ class PSoundChannel : public PChannel
     /**Get the internal buffers for the sound channel I/O. 
 
        @return
-       PTrue if the buffer size were obtained.
+       true if the buffer size were obtained.
      */
     virtual PBoolean GetBuffers(
       PINDEX & size,    // Size of each buffer
@@ -436,10 +489,10 @@ class PSoundChannel : public PChannel
     /**Set the volume of the play/read process.
        The volume range is 0 == quiet, 100 == LOUDEST. The volume is a
        logarithmic scale mapped from the lowest gain possible on the device to
-       the highest gain.
+       the highest gain
         
        @return
-       PTrue if there were no errors.
+       true if there were no errors.
     */
     virtual PBoolean SetVolume(
       unsigned volume   ///< New volume level
@@ -451,7 +504,7 @@ class PSoundChannel : public PChannel
        the highest gain.
 
        @return
-       PTrue if there were no errors.
+       true if there were no errors.
     */
     virtual PBoolean GetVolume(
       unsigned & volume   ///< Variable to receive volume level.
@@ -461,44 +514,84 @@ class PSoundChannel : public PChannel
   /**@name Play functions */
   //@{
 
-    /** Low level write (or play) to the channel. This function will block until the
-       requested number of characters are written or the write timeout is
-       reached. The GetLastWriteCount() function returns the actual number
-       of bytes written.
-                                                                                                                                            
-       The GetErrorCode() function should be consulted after Write() returns
-       PFalse to determine what caused the failure.
+    /** Low level write (or play) to the channel. 
+
+        It will generate a logical assertion if you attempt write to a
+        channel set up for recording.
+
+        @param buf is a pointer to the data to be written to the
+        channel.  It is an error for this pointer to be NULL. A logical
+        assert will be generated when buf is NULL.
+
+        @param len Nr of bytes to send. If len equals the buffer size
+        set by SetBuffers() it will block for
+        (1000*len)/(samplesize*samplerate) ms. Typically, the sample
+        size is 2 bytes.  If len == 0, this will return immediately,
+        where the return value is equal to the value of IsOpen().
  
-       @return
-       PTrue if at least len bytes were written to the channel.
+        @return true if len bytes were written to the channel,
+        otherwise false. The GetErrorCode() function should be 
+        consulted after Write() returns false to determine what 
+        caused the failure.
      */
     virtual PBoolean Write(const void * buf, PINDEX len);
 
+
+    /** Low level write (or play) with watermark to the channel. 
+
+        It will generate a logical assertion if you attempt write to a
+        channel set up for recording.
+
+        @param buf is a pointer to the data to be written to the
+        channel.  It is an error for this pointer to be NULL. A logical
+        assert will be generated when buf is NULL.
+
+        @param len Nr of bytes to send. If len equals the buffer size
+        set by SetBuffers() it will block for
+        (1000*len)/(samplesize*samplerate) ms. Typically, the sample
+        size is 2 bytes.  If len == 0, this will return immediately,
+        where the return value is equal to the value of IsOpen().
+      
+        @param mark Unique identifer to identify the write
+ 
+        @return PTrue if len bytes were written to the channel,
+        otherwise PFalse. The GetErrorCode() function should be 
+        consulted after Write() returns PFalse to determine what 
+        caused the failure.
+     */
+    virtual PBoolean Write(
+      const void * buf,         ///< Pointer to a block of memory to write.
+      PINDEX len,               ///< Number of bytes to write.
+      const void * mark         ///< Unique Marker to identify write
+    );
+
+    /** Get number of bytes written in last Write() operation. */
     virtual PINDEX GetLastWriteCount() const;
 
-    /**Play a sound to the open device. If the #wait# parameter is
-       PTrue then the function does not return until the file has been played.
-       If PFalse then the sound play is begun asynchronously and the function
+    /**Play a sound to the open device. If the <code>wait</code> parameter is
+       true then the function does not return until the file has been played.
+       If false then the sound play is begun asynchronously and the function
        returns immediately.
 
-       Note if the driver is closed of the object destroyed then the sound
-       play is aborted.
+       Note:  if the driver is closed while playing the sound, the play 
+       operation stops immediately.
 
        Also note that not all possible sounds and sound files are playable by
        this library. No format conversions between sound object and driver are
        performed.
 
        @return
-       PTrue if the sound is playing or has played.
+       true if the sound is playing or has played.
      */
 
     virtual PBoolean PlaySound(
       const PSound & sound,   ///< Sound to play.
-      PBoolean wait = PTrue        ///< Flag to play sound synchronously.
+      PBoolean wait = true        ///< Flag to play sound synchronously.
     );
-    /**Play a sound file to the open device. If the #wait#
-       parameter is PTrue then the function does not return until the file has
-       been played. If PFalse then the sound play is begun asynchronously and
+
+    /**Play a sound file to the open device. If the <code>wait</code>
+       parameter is true then the function does not return until the file has
+       been played. If false then the sound play is begun asynchronously and
        the function returns immediately.
 
        Note if the driver is closed of the object destroyed then the sound
@@ -509,26 +602,26 @@ class PSoundChannel : public PChannel
        performed.
 
        @return
-       PTrue if the sound is playing or has played.
+       true if the sound is playing or has played.
      */
     virtual PBoolean PlayFile(
       const PFilePath & file, ///< Sound file to play.
-      PBoolean wait = PTrue        ///< Flag to play sound synchronously.
+      PBoolean wait = true        ///< Flag to play sound synchronously.
     );
 
     /**Indicate if the sound play begun with PlayBuffer() or PlayFile() has
        completed.
 
        @return
-       PTrue if the sound has completed playing.
+       true if the sound has completed playing.
      */
     virtual PBoolean HasPlayCompleted();
 
-    /**Block the thread until the sound play begun with PlayBuffer() or
-       PlayFile() has completed.
+    /**Block calling thread until the sound play begun with PlaySound() or
+       PlayFile() has completed. 
 
        @return
-       PTrue if the sound has successfully completed playing.
+       true if the sound has successfully completed playing.
      */
     virtual PBoolean WaitForPlayCompletion();
 
@@ -541,18 +634,35 @@ class PSoundChannel : public PChannel
        reached. The GetLastReadCount() function returns the actual number
        of bytes read.
 
-       The GetErrorCode() function should be consulted after Read() returns
-       PFalse to determine what caused the failure.
+       It will generate a logical assertion if you attempt to read
+       from a PSoundChannel that is setup for playing.
 
-       @return
-       PTrue indicates that at least one character was read from the channel.
-       PFalse means no bytes were read due to timeout or some other I/O error.
+       The GetErrorCode() function should be consulted after Read() returns
+       false to determine what caused the failure.
+
+       @param len Nr of bytes to endeaveour to read from the sound
+       device. If len equals the buffer size set by SetBuffers() it
+       will block for (1000*len)/(samplesize*samplerate)
+       ms. Typically, the sample size is 2 bytes.  If len == 0, this
+       will return immediately, where the return value is equal to
+       the value of IsOpen().
+
+       @param buf is a pointer to the empty data area, which will
+       contain the data collected from the sound device.  It is an
+       error for this pointer to be NULL. A logical assert will be
+       generated when buf is NULL.
+
+       @return true indicates that at least one character was read
+       from the channel.  false means no bytes were read due to some
+       I/O error, (which includes timeout or some other thread closed
+       the device).
      */
     virtual PBoolean Read(
       void * buf,   ///< Pointer to a block of memory to receive the read bytes.
       PINDEX len    ///< Maximum number of bytes to read into the buffer.
     );
 
+    /** Return number of bytes read in last Read() call. */
     PINDEX GetLastReadCount() const;
 
     /**Record into the sound object all of the buffer's of sound data. Use the
@@ -570,7 +680,7 @@ class PSoundChannel : public PChannel
        without blocking.
 
        @return
-       PTrue if the sound has been recorded.
+       true if the sound has been recorded.
      */
     virtual PBoolean RecordSound(
       PSound & sound ///< Sound recorded
@@ -586,7 +696,7 @@ class PSoundChannel : public PChannel
        without blocking.
 
        @return
-       PTrue if the sound has been recorded.
+       true if the sound has been recorded.
      */
     virtual PBoolean RecordFile(
       const PFilePath & file ///< Sound file recorded
@@ -596,7 +706,7 @@ class PSoundChannel : public PChannel
        initiate the recording.
 
        @return
-       PTrue if the sound driver has successfully started recording.
+       true if the sound driver has successfully started recording.
      */
     virtual PBoolean StartRecording();
 
@@ -605,7 +715,7 @@ class PSoundChannel : public PChannel
        the buffer size.
 
        @return
-       PTrue if the sound driver has filled a buffer.
+       true if the sound driver has filled a buffer.
      */
     virtual PBoolean IsRecordBufferFull();
 
@@ -615,7 +725,7 @@ class PSoundChannel : public PChannel
        a new recording.
 
        @return
-       PTrue if the sound driver has filled a buffer.
+       true if the sound driver has filled a buffer.
      */
     virtual PBoolean AreAllRecordBuffersFull();
 
@@ -624,7 +734,7 @@ class PSoundChannel : public PChannel
        is less than the buffer size.
 
        @return
-       PTrue if the sound driver has filled a buffer.
+       true if the sound driver has filled a buffer.
      */
     virtual PBoolean WaitForRecordBufferFull();
 
@@ -634,13 +744,19 @@ class PSoundChannel : public PChannel
        again to start a new recording.
 
        @return
-       PTrue if the sound driver has filled a buffer.
+       true if the sound driver has filled a buffer.
      */
     virtual PBoolean WaitForAllRecordBuffersFull();
   //@}
 
   protected:
     PSoundChannel * baseChannel;
+
+    /**This is the direction that this sound channel is opened for use
+       in.  Should the user attempt to used this opened class instance
+       in a direction opposite to that specified in activeDirection,
+       an assert happens. */
+    Directions      activeDirection;
 };
 
 
@@ -659,6 +775,22 @@ template <class className> class PSoundChannelPluginServiceDescriptor : public P
   static PSoundChannelPluginServiceDescriptor<className> className##_descriptor; \
   PCREATE_PLUGIN(name, PSoundChannel, &className##_descriptor)
 
+#ifdef _WIN32
+  PPLUGIN_STATIC_LOAD(WindowsMultimedia, PSoundChannel);
+#elif defined(__BEOS__)
+  PPLUGIN_STATIC_LOAD(BeOS, PSoundChannel);
 #endif
 
+#if defined(P_DIRECTSOUND)
+  PPLUGIN_STATIC_LOAD(DirectSound, PSoundChannel);
+#endif
+
+#if defined(P_WAVFILE)
+  PPLUGIN_STATIC_LOAD(WAVFile, PSoundChannel)
+#endif
+
+
+#endif // PTLIB_SOUND_H
+
+
 // End Of File ///////////////////////////////////////////////////////////////
