@@ -30,9 +30,9 @@
  * Contributor(s): Guilhem Tardy (gtardy@salyens.com)
  *  Nicola Orru' <nigu@itadinanta.it>
  *
- * $Revision: 26249 $
+ * $Revision: 26398 $
  * $Author: ededu $
- * $Date: 2011-07-26 09:03:18 -0500 (Tue, 26 Jul 2011) $
+ * $Date: 2011-09-06 04:15:28 -0500 (Tue, 06 Sep 2011) $
  */
 
 #pragma implementation "vidinput_v4l2.h"
@@ -373,20 +373,15 @@ PBoolean PVideoInputDevice_V4L2::SetVideoFormat(VideoFormat newFormat)
 }
 
 
-int PVideoInputDevice_V4L2::GetNumChannels() 
+int PVideoInputDevice_V4L2::GetNumChannels()
 {
   // if opened, return the capability value, else 1 as in videoio.cxx
   if (IsOpen ()) {
 
     struct v4l2_input videoEnumInput;
     videoEnumInput.index = 0;
-    while (1) {
-      if (v4l2_ioctl(videoFd, VIDIOC_ENUMINPUT, &videoEnumInput) < 0) {
-        break;
-      }
-      else
-        videoEnumInput.index++;
-    }
+    while (v4l2_ioctl(videoFd, VIDIOC_ENUMINPUT, &videoEnumInput) >= 0)
+      videoEnumInput.index++;
 
     return videoEnumInput.index;
   }
@@ -404,7 +399,7 @@ PBoolean PVideoInputDevice_V4L2::SetChannel(int newChannel)
 
   // set the channel
   if (v4l2_ioctl(videoFd, VIDIOC_S_INPUT, &channelNumber) < 0) {
-    PTRACE(1,"VideoInputDevice\tS_INPUT failed : " << ::strerror(errno));    
+    PTRACE(1,"VideoInputDevice\tS_INPUT failed : " << ::strerror(errno));
     return PFalse;
   }
 
