@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24412 $
+ * $Revision: 26635 $
  * $Author: rjongbloed $
- * $Date: 2010-05-27 18:53:50 -0500 (Thu, 27 May 2010) $
+ * $Date: 2011-11-03 01:23:34 -0500 (Thu, 03 Nov 2011) $
  */
 
 //////////////////////////////////////////////////
@@ -39,8 +39,7 @@
 #include <ptclib/pstun.h>
 
 
-static const char FactoryName[] = PINTERFACE_MONITOR_FACTORY_NAME;
-static PFactory<PProcessStartup>::Worker<PInterfaceMonitor> InterfaceMonitorFactory(FactoryName, true);
+PFACTORY_CREATE_SINGLETON(PProcessStartupFactory, PInterfaceMonitor);
 
 #ifndef _WIN32_WCE
 #define UDP_BUFFER_SIZE 32768
@@ -99,12 +98,6 @@ PInterfaceMonitor::~PInterfaceMonitor()
 }
 
 
-PInterfaceMonitor & PInterfaceMonitor::GetInstance()
-{
-  return *PFactory<PProcessStartup>::CreateInstanceAs<PInterfaceMonitor>(FactoryName);
-}
-
-
 void PInterfaceMonitor::SetRefreshInterval(unsigned refresh)
 {
   m_refreshInterval = refresh;
@@ -1023,6 +1016,11 @@ PChannel::Errors PMonitoredSocketBundle::ReadFromBundle(void * buf,
 void PMonitoredSocketBundle::OnAddInterface(const InterfaceEntry & entry)
 {
   // Already locked
+
+  // Temporarily ignore IPv6 interfaces, will be available in Eridani (> v2.11)
+  if (entry.GetAddress().GetVersion() == 6)
+    return;
+
   if (opened) {
     OpenSocket(MakeInterfaceDescription(entry));
     PTRACE(3, "MonSock\tUDP socket bundle has added interface " << entry);
@@ -1169,6 +1167,10 @@ void PSingleMonitoredSocket::OnAddInterface(const InterfaceEntry & entry)
 {
   // Already locked
 
+  // Temporarily ignore IPv6 interfaces, will be available in Eridani (> v2.11)
+  if (entry.GetAddress().GetVersion() == 6)
+    return;
+
   PIPSocket::Address addr;
   PString name;
   if (!SplitInterfaceDescription(theInterface, addr, name))
