@@ -27,9 +27,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26303 $
+ * $Revision: 26804 $
  * $Author: rjongbloed $
- * $Date: 2011-08-12 03:44:54 -0500 (Fri, 12 Aug 2011) $
+ * $Date: 2011-12-17 01:23:10 -0600 (Sat, 17 Dec 2011) $
  */
 
 #ifdef __GNUC__
@@ -290,10 +290,15 @@ PBoolean PWAVFile::RawRead(void * buf, PINDEX len)
   // We do not want to return this data by mistake.
   PINDEX readlen = len;
   off_t pos = PFile::GetPosition();
-  if (pos >= (lenHeader+lenData))
+
+  // indicate eof (return false, but error=0, lastReadCount=0)
+  if (pos >= (lenHeader + lenData)) {
+    lastReadCount = 0;
+    ConvertOSError(0, LastReadError);
     return PFalse;
-  
-  if ((pos + len) > (lenHeader+lenData))
+  }
+
+  if ((pos + len) > (lenHeader + lenData))
     readlen = (lenHeader+lenData) - pos;
 
   if (formatHandler != NULL)
