@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24060 $
+ * $Revision: 26981 $
  * $Author: rjongbloed $
- * $Date: 2010-02-12 23:11:44 -0600 (Fri, 12 Feb 2010) $
+ * $Date: 2012-02-13 18:28:48 -0600 (Mon, 13 Feb 2012) $
  */
 
 #include <ptlib.h>
@@ -68,6 +68,7 @@ PContainer::PContainer(PINDEX initialSize)
   PAssert(reference != NULL, POutOfMemory);
 }
 
+
 PContainer::PContainer(int, const PContainer * cont)
 {
   if (cont == this)
@@ -80,6 +81,7 @@ PContainer::PContainer(int, const PContainer * cont)
   PAssert(reference != NULL, POutOfMemory);
 }
 
+
 PContainer::PContainer(const PContainer & cont)
 {
   if (&cont == this)
@@ -92,27 +94,32 @@ PContainer::PContainer(const PContainer & cont)
 }
 
 
+PContainer::PContainer(PContainerReference & ref)
+  : reference(&ref)
+{
+}
+
+
 void PContainer::AssignContents(const PContainer & cont)
 {
   if(cont.reference == NULL){
     PAssertAlways("container reference is null");
     return;
-  } else if(cont.GetClass() == NULL){
+  }
+  if(cont.GetClass() == NULL){
     PAssertAlways("container class is null");
     return;
   }
 
-  if (reference == cont.reference) {
+  if (reference == cont.reference)
     return;
-  }
 
   if (--reference->count == 0) {
     DestroyContents();
-    delete reference;
-    reference = NULL;
+    DestroyReference();
   }
 
-  ++cont.reference->count;
+  PAssert(++cont.reference->count > 1, "Assignment of container that was deleted");
   reference = cont.reference;
 }
 
@@ -120,19 +127,21 @@ void PContainer::AssignContents(const PContainer & cont)
 void PContainer::Destruct()
 {
   if (reference != NULL) {
-    if (--reference->count > 0) {
-      reference = NULL;
-    }
-    
-    else {
+    if (--reference->count <= 0) {
       DestroyContents();
-      delete reference;
-      reference = NULL;
+      DestroyReference();
     }
+    reference = NULL;
   }
 }
 
 
+void PContainer::DestroyReference()
+{
+  delete reference;
+}
+
+
 PBoolean PContainer::SetMinSize(PINDEX minSize)
 {
   PASSERTINDEX(minSize);
@@ -203,6 +212,15 @@ PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes,
 }
 
 
+PAbstractArray::PAbstractArray(PContainerReference & reference, PINDEX elementSizeInBytes)
+  : PContainer(reference)
+  , elementSize(elementSizeInBytes)
+  , theArray(NULL)
+  , allocatedDynamically(false)
+{
+}
+
+
 void PAbstractArray::DestroyContents()
 {
   if (theArray != NULL) {
@@ -218,6 +236,9 @@ void PAbstractArray::CopyContents(const PAbstractArray & array)
   elementSize = array.elementSize;
   theArray = array.theArray;
   allocatedDynamically = array.allocatedDynamically;
+
+  if (reference->constObject)
+    MakeUnique();
 }
 
 
@@ -325,6 +346,8 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
       if ((newArray = PAbstractArray_allocator.allocate(newsizebytes)) == NULL)
         return PFalse;
   
+      allocatedDynamically = true;
+
       if (theArray != NULL)
         memcpy(newArray, theArray, PMIN(oldsizebytes, newsizebytes));
     }
@@ -592,7 +615,7 @@ PBoolean PBitArray::Concatenate(const PBitArray & array)
 ///////////////////////////////////////////////////////////////////////////////
 
 PString::PString(const char * cstr)
-  : PCharArray(cstr != NULL ? strlen(cstr)+1 : 1)
+  : PCharArray(cstr != NULL ? (int)strlen(cstr)+1 : 1)
 {
   if (cstr != NULL)
     memcpy(theArray, cstr, GetSize());
@@ -944,22 +967,25 @@ void PString::PrintOn(ostream &strm) const
 
 void PString::ReadFrom(istream &strm)
 {
-  SetMinSize(100);
-  char * ptr = theArray;
+  PINDEX bump = 16;
   PINDEX len = 0;
-  while (strm.peek() != EOF) {
-    if ((*ptr = (char)strm.get()) == '\n')
-      break;
-    ptr++;
-    len++;
-    if (len >= GetSize()) {
-      SetSize(len + 100);
-      ptr = theArray + len;
+  do {
+    if (!SetMinSize(len + (bump *= 2))) {
+      strm.setstate(ios::badbit);
+      return;
     }
-  }
-  *ptr = '\0';
-  if ((len > 0) && (ptr[-1] == '\r'))
-    ptr[-1] = '\0';
+
+    strm.clear();
+    strm.getline(theArray + len, GetSize() - len);
+    len += (PINDEX)strm.gcount();
+  } while (strm.fail() && !strm.eof());
+
+  if (len > 0 && !strm.eof())
+    --len; // Allow for extracted '\n'
+
+  if (len > 0 && theArray[len-1] == '\r')
+    theArray[len-1] = '\0';
+
   PAssert(MakeMinimumSize(), POutOfMemory);
 }
 
@@ -992,7 +1018,12 @@ PBoolean PString::IsEmpty() const
 
 PBoolean PString::SetSize(PINDEX newSize)
 {
-  return InternalSetSize(newSize, PTrue);
+  if (newSize < 1)
+    newSize = 1;
+  if (!InternalSetSize(newSize, true))
+    return false;
+  theArray[newSize-1] = '\0';
+  return true;
 }
 
 
@@ -1845,6 +1876,8 @@ PString PString::ToLiteral() const
   for (char * p = theArray; *p != '\0'; p++) {
     if (*p == '"')
       str += "\\\"";
+    else if (*p == '\\')
+      str += "\\\\";
     else if (isprint(*p & 0xff))
       str += *p;
     else {
@@ -2570,6 +2603,56 @@ char ** PStringToString::ToCharArray(bool withEqualSign, PCharArray * storage) c
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+
+
+PString PStringOptions::GetString(const PCaselessString & key, const char * dflt) const
+{
+  PString * str = PStringToString::GetAt(key);
+  return str != NULL ? *str : PString(dflt);
+}
+
+
+bool PStringOptions::GetBoolean(const PCaselessString & key, bool dflt) const
+{
+  PString * str = PStringToString::GetAt(key);
+  if (str == NULL)
+    return dflt;
+
+  if (str->IsEmpty() || str->AsUnsigned() != 0)
+    return true;
+
+  PCaselessString test(*str);
+  return test.NumCompare("t") == EqualTo || test.NumCompare("y") == EqualTo;
+}
+
+
+long PStringOptions::GetInteger(const PCaselessString & key, long dflt) const
+{
+  PString * str = PStringToString::GetAt(key);
+  return str != NULL ? str->AsInteger() : dflt;
+}
+
+
+void PStringOptions::SetInteger(const PCaselessString & key, long value)
+{
+  PStringToString::SetAt(key, PString(PString::Signed, value));
+}
+
+
+double PStringOptions::GetReal(const PCaselessString & key, double dflt) const
+{
+  PString * str = PStringToString::GetAt(key);
+  return str != NULL ? str->AsReal() : dflt;
+}
+
+
+void PStringOptions::SetReal(const PCaselessString & key, double value, int decimals)
+{
+  PStringToString::SetAt(key, PString(decimals < 0 ? PString::Exponent : PString::Decimal, value, decimals));
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PRegularExpression::PRegularExpression()
@@ -2624,6 +2707,12 @@ PRegularExpression::~PRegularExpression()
 }
 
 
+void PRegularExpression::PrintOn(ostream &strm) const
+{
+  strm << patternSaved;
+}
+
+
 PRegularExpression::ErrorCodes PRegularExpression::GetErrorCode() const
 {
   return lastError;
@@ -2739,27 +2828,51 @@ PBoolean PRegularExpression::Execute(const char * cstr,
     return PFalse;
   }
 
-  regmatch_t single_match;
-  regmatch_t * matches = &single_match;
-
   PINDEX count = starts.GetSize();
-  if (count > 1)
-    matches = new regmatch_t[count];
-  else
+  if (count == 0) {
+    starts.SetSize(1);
     count = 1;
+  }
+  ends.SetSize(count);
 
-  lastError = (ErrorCodes)regexec(regexpression(), cstr, count, matches, flags);
+  regmatch_t * matches = new regmatch_t[count];
+
+  lastError = (ErrorCodes)::regexec(regexpression(), cstr, count, matches, flags);
   if (lastError == NoError) {
-    starts.SetMinSize(count);
-    ends.SetMinSize(count);
     for (PINDEX i = 0; i < count; i++) {
       starts[i] = matches[i].rm_so;
       ends[i] = matches[i].rm_eo;
     }
   }
 
-  if (matches != &single_match)
-    delete [] matches;
+  delete [] matches;
+
+  return lastError == NoError;
+}
+
+
+PBoolean PRegularExpression::Execute(const char * cstr, PStringArray & substring, int flags) const
+{
+  if (expression == NULL) {
+    lastError = NotCompiled;
+    return PFalse;
+  }
+
+  PINDEX count = substring.GetSize();
+  if (count == 0) {
+    substring.SetSize(1);
+    count = 1;
+  }
+
+  regmatch_t * matches = new regmatch_t[count];
+
+  lastError = (ErrorCodes)::regexec(regexpression(), cstr, count, matches, flags);
+  if (lastError == NoError) {
+    for (PINDEX i = 0; i < count; i++)
+      substring[i] = PString(cstr+matches[i].rm_so, matches[i].rm_eo-matches[i].rm_so);
+  }
+
+  delete [] matches;
 
   return lastError == NoError;
 }
