@@ -26,20 +26,22 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26203 $
+ * $Revision: 26958 $
  * $Author: rjongbloed $
- * $Date: 2011-07-14 23:36:07 -0500 (Thu, 14 Jul 2011) $
+ * $Date: 2012-02-08 15:46:44 -0600 (Wed, 08 Feb 2012) $
  */
 
 #include <ptlib.h>
 #include <vector>
 #include <map>
 #include <fstream>
+#include <algorithm>
 
 #include <ctype.h>
 #include <ptlib/pfactory.h>
 #include <ptlib/pprocess.h>
 #include <ptlib/svcproc.h>
+#include <ptlib/pluginmgr.h>
 #include "../../../version.h"
 #include "../../../revision.h"
 
@@ -825,6 +827,10 @@ void PTimer::Stop(bool wait)
     m_state = Stopped;
     m_timerList->QueueRequest(PTimerList::RequestType::Stop, this, wait);
   }
+  else if (wait) {
+    // ensure that timer is stopped correctly
+    m_timerList->QueueRequest(PTimerList::RequestType::Stop, this, true);
+  }
 }
 
 
@@ -1516,8 +1522,6 @@ PString PConfigArgs::CharToString(char ch) const
 
 PProcess * PProcessInstance;
 
-typedef std::map<PString, PProcessStartup *> PProcessStartupList;
-
 int PProcess::InternalMain(void *)
 {
   Main();
@@ -1545,13 +1549,6 @@ void PProcess::PreInitialise(int c, char ** v, char **)
 }
 
 
-static PProcessStartupList & GetPProcessStartupList()
-{
-  static PProcessStartupList list;
-  return list;
-}
-
-
 PProcess::PProcess(const char * manuf, const char * name,
                    WORD major, WORD minor, CodeStatus stat, WORD build,
                    bool library)
@@ -1568,6 +1565,7 @@ PProcess::PProcess(const char * manuf, const char * name,
 {
   m_activeThreads[GetCurrentThreadId()] = this;
 
+  PAssert(PProcessInstance == NULL, "Only one instance of PProcess allowed");
   PProcessInstance = this;
 
 #ifdef P_RTEMS
@@ -1591,23 +1589,16 @@ PProcess::PProcess(const char * manuf, const char * name,
 
   Construct();
 
-  // create one instance of each class registered in the 
-  // PProcessStartup abstract factory
-  PProcessStartupList & startups = GetPProcessStartupList();
-  {
-    PProcessStartup * levelSet = PFactory<PProcessStartup>::CreateInstance("SetTraceLevel");
-    if (levelSet != NULL) 
-      levelSet->OnStartup();
-
-    PProcessStartupFactory::KeyList_T list = PProcessStartupFactory::GetKeyList();
-    PProcessStartupFactory::KeyList_T::const_iterator r;
-    for (r = list.begin(); r != list.end(); ++r) {
-      if (*r != "SetTraceLevel") {
-        PProcessStartup * instance = PProcessStartupFactory::CreateInstance(*r);
-        instance->OnStartup();
-        startups.insert(std::pair<PString, PProcessStartup *>(*r, instance));
-      }
-    }
+  // create one instance of each class registered in the PProcessStartup abstract factory
+  // But make sure we have plugins first, to avoid bizarre behaviour where static objects
+  // are initialised multiple times when libraries are loaded in Linux.
+  PProcessStartupFactory::KeyList_T list = PProcessStartupFactory::GetKeyList();
+  std::swap(list.front(), *std::find(list.begin(), list.end(), PLUGIN_LOADER_STARTUP_NAME));
+  list.insert(list.begin(), "SetTraceLevel");
+  for (PProcessStartupFactory::KeyList_T::const_iterator it = list.begin(); it != list.end(); ++it) {
+    PProcessStartup * startup = PProcessStartupFactory::CreateInstance(*it);
+    if (startup != NULL)
+      startup->OnStartup();
   }
 
 #if PMEMORY_HEAP
@@ -1620,19 +1611,18 @@ PProcess::PProcess(const char * manuf, const char * name,
 void PProcess::PreShutdown()
 {
   PProcessInstance->m_shuttingDown = true;
-  PProcessStartupList & startups = GetPProcessStartupList();
-  for (PProcessStartupList::iterator startup = startups.begin(); startup != startups.end(); ++startup)
-    startup->second->OnShutdown();
+  PProcessStartupFactory::KeyList_T list = PProcessStartupFactory::GetKeyList();
+  for (PProcessStartupFactory::KeyList_T::const_iterator it = list.begin(); it != list.end(); ++it)
+    PProcessStartupFactory::CreateInstance(*it)->OnShutdown();
 }
 
 
 void PProcess::PostShutdown()
 {
-  PProcessStartupList & startups = GetPProcessStartupList();
-  for (PProcessStartupList::iterator startup = startups.begin(); startup != startups.end(); ++startup)
-    delete startup->second;
-
-  startups.clear();
+  PWaitAndSignal mutex(PFactoryBase::GetFactoriesMutex());
+  PFactoryBase::FactoryMap & factories = PFactoryBase::GetFactories();
+  for (PFactoryBase::FactoryMap::iterator it = factories.begin(); it != factories.end(); ++it)
+    it->second->DestroySingletons();
 
   PProcessInstance = NULL;
 }
@@ -2285,7 +2275,7 @@ void PReadWriteMutex::InternalWait(PSemaphore & semaphore) const
   ostream & trace = PTrace::Begin(1, __FILE__, __LINE__);
   trace << "PTLib\tPossible deadlock in read/write mutex:\n";
   for (std::map<PThreadIdentifier, Nest>::const_iterator it = m_nestedThreads.begin(); it != m_nestedThreads.end(); ++it)
-    trace << "  thread-id=" << it->first << ","
+    trace << "  thread-id=" << it->first << " (0x" << std::hex << it->first << std::dec << "),"
               " readers=" << it->second.readerCount << ","
               " writers=" << it->second.writerCount << '\n';
   trace << PTrace::End;
