@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24265 $
- * $Author: rjongbloed $
- * $Date: 2010-04-22 23:49:18 -0500 (Thu, 22 Apr 2010) $
+ * $Revision: 26309 $
+ * $Author: ededu $
+ * $Date: 2011-08-14 18:01:01 -0500 (Sun, 14 Aug 2011) $
  */
 
 #include <ptlib.h>
@@ -85,7 +85,7 @@ static int defaultIpAddressFamily = PF_INET;  // PF_UNSPEC;   // default to IPV4
 
 static PIPSocket::Address loopback4(127,0,0,1);
 static PIPSocket::Address broadcast4(INADDR_BROADCAST);
-static PIPSocket::Address any4(INADDR_ANY);
+static PIPSocket::Address any4(0,0,0,0);
 static in_addr inaddr_empty;
 #if P_HAS_IPV6
 static int defaultIPv6ScopeId = 0; 
@@ -545,6 +545,10 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
     struct addrinfo *res = NULL;
     struct addrinfo hints = { AI_CANONNAME, defaultIpAddressFamily };
     localErrNo = getaddrinfo((const char *)name, NULL , &hints, &res);
+    if (localErrNo != 0 && defaultIpAddressFamily == AF_INET6) {
+      hints.ai_family = AF_INET;
+      localErrNo = getaddrinfo((const char *)name, NULL , &hints, &res);
+    }
     host = new PIPCacheData(localErrNo != NETDB_SUCCESS ? NULL : res, name);
     if (res != NULL)
       freeaddrinfo(res);
@@ -1619,7 +1623,7 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
 
 #if P_HAS_IPV6
 
-/// Check for v4 mapped i nv6 address ::ffff:a.b.c.d
+/// Check for v4 mapped in v6 address ::ffff:a.b.c.d
 PBoolean PIPSocket::Address::IsV4Mapped() const
 {
   if (version != 6)
@@ -1627,6 +1631,14 @@ PBoolean PIPSocket::Address::IsV4Mapped() const
   return IN6_IS_ADDR_V4MAPPED(&v.six) || IN6_IS_ADDR_V4COMPAT(&v.six);
 }
 
+/// Check for link-local address fe80::/10
+PBoolean PIPSocket::Address::IsLinkLocal() const
+{
+  if (version != 6)
+    return PFalse;
+  return IN6_IS_ADDR_LINKLOCAL(&v.six);
+}
+
 #endif
 
 const PIPSocket::Address & PIPSocket::Address::GetLoopback(int IPV6_PARAM(version))
@@ -2130,18 +2142,11 @@ PIPSocket::InterfaceEntry::InterfaceEntry()
 PIPSocket::InterfaceEntry::InterfaceEntry(const PString & _name,
                                           const Address & _addr,
                                           const Address & _mask,
-                                          const PString & _macAddr
-#if P_HAS_IPV6
-                                         ,const PString & _ip6Addr
-#endif
-                                         )
+                                          const PString & _macAddr)
   : name(_name.Trim()),
     ipAddr(_addr),
     netMask(_mask),
     macAddr(_macAddr)
-#if P_HAS_IPV6
-    , ip6Addr(_ip6Addr)
-#endif
 {
 }
 
@@ -2149,10 +2154,6 @@ PIPSocket::InterfaceEntry::InterfaceEntry(const PString & _name,
 void PIPSocket::InterfaceEntry::PrintOn(ostream & strm) const
 {
   strm << ipAddr;
-#if P_HAS_IPV6
-  if (!ip6Addr)
-    strm << " [" << ip6Addr << ']';
-#endif
   if (!macAddr)
     strm << " <" << macAddr << '>';
   if (!name)
@@ -2422,23 +2423,21 @@ PBoolean PIPDatagramSocket::ReadFrom(void * buf, PINDEX len,
 
   Psockaddr sa;
   PINDEX size = sa.GetSize();
-  if (os_recvfrom(buf, len, 0, sa, &size)) {
-    addr = sa.GetIP();
-    port = sa.GetPort();
-  }
+  bool ok = os_recvfrom(buf, len, 0, sa, &size);
+  addr = sa.GetIP();
+  port = sa.GetPort();
 
 #else
 
   sockaddr_in sockAddr;
   PINDEX addrLen = sizeof(sockAddr);
-  if (os_recvfrom(buf, len, 0, (struct sockaddr *)&sockAddr, &addrLen)) {
-    addr = sockAddr.sin_addr;
-    port = ntohs(sockAddr.sin_port);
-  }
+  bool ok = os_recvfrom(buf, len, 0, (struct sockaddr *)&sockAddr, &addrLen);
+  addr = sockAddr.sin_addr;
+  port = ntohs(sockAddr.sin_port);
 
 #endif
 
-  return lastReadCount > 0;
+  return ok;
 }
 
 
@@ -2720,67 +2719,39 @@ PBoolean PUDPSocket::OpenSocketGQOS(int af, int type, int proto)
 {
 #if defined(_WIN32) && defined(P_QOS)
     
-  DWORD bufferSize = 0;
-  DWORD numProtocols, i;
-  LPWSAPROTOCOL_INFO installedProtocols, qosProtocol;
-
   //Try to find a QOS-enabled protocol
- 
-  PBoolean retval = ConvertOSError(numProtocols = WSAEnumProtocols(((proto==0) ? NULL : &proto),
-                                                            NULL,
-                                                            &bufferSize));
-    
-  if (numProtocols == SOCKET_ERROR && WSAGetLastError()!=WSAENOBUFS) 
-    return retval;
+  DWORD bufferSize = 0;
+  DWORD numProtocols = WSAEnumProtocols(proto != 0 ? &proto : NULL, NULL, &bufferSize);
+  if (!ConvertOSError(numProtocols) && WSAGetLastError() != WSAENOBUFS) 
+    return false;
 
-  installedProtocols = (LPWSAPROTOCOL_INFO)(new BYTE[bufferSize]);
-  retval = ConvertOSError(numProtocols = WSAEnumProtocols(((proto==0) ? NULL : &proto),
-                                                            installedProtocols,
-                                                            &bufferSize));
-  if (numProtocols == SOCKET_ERROR) {
+  LPWSAPROTOCOL_INFO installedProtocols = (LPWSAPROTOCOL_INFO)(new BYTE[bufferSize]);
+  numProtocols = WSAEnumProtocols(proto != 0 ? &proto : NULL, installedProtocols, &bufferSize);
+  if (!ConvertOSError(numProtocols)) {
     delete[] installedProtocols;
-    return retval;
+    return false;
   }
 
-  qosProtocol = installedProtocols;
-  PBoolean haveQoSproto = PFalse;
-
-  for (i=0; i<numProtocols; qosProtocol++, i++) {
+  LPWSAPROTOCOL_INFO qosProtocol = installedProtocols;
+  for (DWORD i = 0; i < numProtocols; qosProtocol++, i++) {
     if ((qosProtocol->dwServiceFlags1 & XP1_QOS_SUPPORTED) &&
         (qosProtocol->iSocketType == type) &&
         (qosProtocol->iAddressFamily == af)) {
-      haveQoSproto = PTrue;
+      os_handle = WSASocket(af, type, proto, qosProtocol, 0, WSA_FLAG_OVERLAPPED);
       break;
     }
   }
 
-  if (haveQoSproto) {
-    retval =  ConvertOSError(os_handle = WSASocket(af,
-                                                   type,
-                                                   proto,
-                                                   qosProtocol,
-                                                   0,
-                                                   WSA_FLAG_OVERLAPPED));
-  }
-  else
-  {    
-    retval = ConvertOSError(os_handle = WSASocket (af,
-                                                   type,
-                                                   proto,
-                                                   NULL,
-                                                   0,
-                                                   WSA_FLAG_OVERLAPPED));
-  }
-
   delete[] installedProtocols;
 
-  if (os_handle == INVALID_SOCKET)
-    return retval;
-#else
-  PBoolean retval = ConvertOSError(os_handle = os_socket(af, type, proto));
-#endif
+  if (!IsOpen())
+    os_handle = WSASocket(af, type, proto, NULL, 0, WSA_FLAG_OVERLAPPED);
 
-  return retval;
+  return ConvertOSError(os_handle);
+
+#else
+  return ConvertOSError(os_handle = os_socket(af, type, proto));
+#endif
 }
 
 #ifdef _WIN32
@@ -2860,24 +2831,42 @@ void PUDPSocket::SetSendAddress(const Address & newAddress, WORD newPort)
 }
 
 
-void PUDPSocket::GetSendAddress(Address & address, WORD & port)
+void PUDPSocket::GetSendAddress(Address & address, WORD & port) const
 {
   address = sendAddress;
   port    = sendPort;
 }
 
 
-void PUDPSocket::GetLastReceiveAddress(Address & address, WORD & port)
+PString PUDPSocket::GetSendAddress() const
+{
+  return sendAddress.AsString(true) + psprintf(":%u", sendPort);
+}
+
+
+void PUDPSocket::GetLastReceiveAddress(Address & address, WORD & port) const
 {
   address = lastReceiveAddress;
   port    = lastReceivePort;
 }
 
+
+PString PUDPSocket::GetLastReceiveAddress() const
+{
+  return lastReceiveAddress.AsString(true) + psprintf(":%u", lastReceivePort);
+}
+
+
 PBoolean PUDPSocket::IsAlternateAddress(const Address &, WORD)
 {
   return false;
 }
 
+PBoolean PUDPSocket::DoPseudoRead(int & /*selectStatus*/)
+{
+   return false;
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
 PBoolean PICMPSocket::OpenSocket(int)
@@ -2889,21 +2878,17 @@ PBoolean PICMPSocket::OpenSocket(int)
 
 PBoolean PIPSocketAddressAndPort::Parse(const PString & str, WORD port, char separator)
 {
-  m_separator= separator;
-  m_port = port;
+  if (separator != '\0')
+    m_separator = separator;
 
-  PINDEX pos = str.Find(separator);
-  if (pos != P_MAX_INDEX) {
-    m_port    = (WORD)str.Mid(pos+1).AsInteger();
-    if (!PIPSocket::GetHostAddress(str.Left(pos), m_address))
-      return false;
-  }
-  else if (m_port == 0) 
-    return false;
-  else if (!PIPSocket::GetHostAddress(str, m_address))
-    return false;
+  PINDEX pos = str.Find(m_separator);
+  if (pos != P_MAX_INDEX)
+    port = (WORD)str.Mid(pos+1).AsUnsigned();
 
-  return true;
+  if (port != 0)
+    m_port = port;
+
+  return PIPSocket::GetHostAddress(str.Left(pos), m_address) && m_port != 0;
 }
 
 
