@@ -24,9 +24,9 @@
  * Contributor(s): Craig Southeren
  *                 Snark at GnomeMeeting
  *
- * $Revision: 21192 $
+ * $Revision: 26804 $
  * $Author: rjongbloed $
- * $Date: 2008-09-30 03:56:36 +0000 (Tue, 30 Sep 2008) $
+ * $Date: 2011-12-17 01:23:10 -0600 (Sat, 17 Dec 2011) $
  */
 
 #ifdef __GNUC__
@@ -40,19 +40,14 @@
 
 static const char soundPluginBaseClass[] = "PSoundChannel";
 
-PINSTANTIATE_FACTORY(PSoundChannel, Win32)
-#ifdef P_DIRECTSOUND
-PINSTANTIATE_FACTORY(PSoundChannel, DirectSound)
-#endif
 
 template <> PSoundChannel * PDevicePluginFactory<PSoundChannel>::Worker::Create(const PString & type) const
 {
   return PSoundChannel::CreateChannel(type);
 }
 
-namespace PWLib {
-  PFactory<PDevicePluginAdapterBase>::Worker< PDevicePluginAdapter<PSoundChannel> > soundChannelFactoryAdapter("PSoundChannel", PTrue);
-};
+typedef PDevicePluginAdapter<PSoundChannel> PDevicePluginSoundChannel;
+PFACTORY_CREATE(PFactory<PDevicePluginAdapterBase>, PDevicePluginSoundChannel, "PSoundChannel", true);
 
 
 PStringArray PSoundChannel::GetDriverNames(PPluginManager * pluginMgr)
@@ -160,14 +155,21 @@ PString PSoundChannel::GetDefaultDevice(Directions dir)
 #endif
 
   PStringArray devices = GetDeviceNames(dir);
-  if (devices.GetSize() > 0)
-    return devices[0];
 
-  return PString::Empty();
+  if (devices.GetSize() == 0)
+    return PString::Empty();
+
+  for (PINDEX i = 0; i < devices.GetSize(); ++i) {
+    if (!(devices[i] *= "NULL"))
+      return devices[i];
+  }
+
+  return devices[0];
 }
 
 
 PSoundChannel::PSoundChannel()
+: activeDirection(Closed)
 {
   baseChannel = NULL;
 }
@@ -203,6 +205,7 @@ PBoolean PSoundChannel::Open(const PString & devSpec,
   }
 
   delete baseChannel;
+  activeDirection = dir;
 
   baseChannel = CreateOpenedChannel(driver, device, dir, numChannels, sampleRate, bitsPerSample);
   if (baseChannel == NULL && !driver.IsEmpty())
@@ -213,172 +216,195 @@ PBoolean PSoundChannel::Open(const PString & devSpec,
 
 PString PSoundChannel::GetName() const
 {
-  if (baseChannel == NULL)
-    return PString::Empty();
-
-  return baseChannel->GetName();
+  return baseChannel != NULL ? baseChannel->GetName() : PString::Empty();
 }
 
 
 PBoolean PSoundChannel::IsOpen() const
 {
-    return baseChannel == NULL || baseChannel->PChannel::IsOpen();
+  return baseChannel != NULL && baseChannel->PChannel::IsOpen();
 }
 
 
 PBoolean PSoundChannel::Close()
 {
-    return baseChannel == NULL || baseChannel->Close();
+  return baseChannel == NULL || baseChannel->Close();
 }
 
 
 int PSoundChannel::GetHandle() const
 {
-    return baseChannel == NULL ? -1 : baseChannel->PChannel::GetHandle();
+  return baseChannel == NULL ? -1 : baseChannel->PChannel::GetHandle();
 }
 
 
 PBoolean PSoundChannel::Abort()
 {
-    return baseChannel == NULL || baseChannel->Abort();
+  PAssert(IsOpen(), PLogicError); 
+  return baseChannel->Abort();
 }
 
 
 PBoolean PSoundChannel::SetFormat(unsigned numChannels, unsigned sampleRate, unsigned bitsPerSample)
 {
-    return baseChannel != NULL && baseChannel->SetFormat(numChannels, sampleRate, bitsPerSample);
+  return baseChannel != NULL && baseChannel->SetFormat(numChannels, sampleRate, bitsPerSample);
 }
 
 
 unsigned PSoundChannel::GetChannels() const
 {
-    return baseChannel == NULL ? 0 : baseChannel->GetChannels();
+  return baseChannel == NULL ? 0 : baseChannel->GetChannels();
 }
 
 
 unsigned PSoundChannel::GetSampleRate() const
 {
-    return baseChannel == NULL ? 0 : baseChannel->GetSampleRate();
+  return baseChannel == NULL ? 0 : baseChannel->GetSampleRate();
 }
 
 
 unsigned PSoundChannel::GetSampleSize() const 
 {
-    return baseChannel == NULL ? 0 : baseChannel->GetSampleSize();
+  return baseChannel == NULL ? 0 : baseChannel->GetSampleSize();
 }
 
 
 PBoolean PSoundChannel::SetBuffers(PINDEX size, PINDEX count)
 {
-    return baseChannel != NULL && baseChannel->SetBuffers(size, count);
+  return baseChannel != NULL && baseChannel->SetBuffers(size, count);
 }
 
 
 PBoolean PSoundChannel::GetBuffers(PINDEX & size, PINDEX & count)
 {
-    return baseChannel != NULL && baseChannel->GetBuffers(size, count);
+  return baseChannel != NULL && baseChannel->GetBuffers(size, count);
 }
 
 
 PBoolean PSoundChannel::SetVolume(unsigned volume)
 {
-    return baseChannel != NULL && baseChannel->SetVolume(volume);
+  return baseChannel != NULL && baseChannel->SetVolume(volume);
 }
 
 
 PBoolean PSoundChannel::GetVolume(unsigned & volume)
 {
-    return baseChannel != NULL && baseChannel->GetVolume(volume);
+  return baseChannel != NULL && baseChannel->GetVolume(volume);
 }
 
 
 PBoolean PSoundChannel::Write(const void * buf, PINDEX len)
 {
-    return baseChannel != NULL && baseChannel->Write(buf, len);
+  PAssert(activeDirection == Player, PLogicError);
+
+  if (len == 0)
+    return IsOpen();
+
+  return baseChannel != NULL && baseChannel->Write(buf, len);
 }
 
+PBoolean PSoundChannel::Write(const void * buf, PINDEX len, const void * /*mark*/)
+{
+  return Write(buf, len);
+}
 
 PINDEX PSoundChannel::GetLastWriteCount() const
 {
-    return baseChannel == NULL ? lastWriteCount : baseChannel->GetLastWriteCount();
+  return baseChannel != NULL ? baseChannel->GetLastWriteCount() : PChannel::GetLastWriteCount();
 }
 
 
 PBoolean PSoundChannel::PlaySound(const PSound & sound, PBoolean wait)
 {
-    return baseChannel != NULL && baseChannel->PlaySound(sound, wait);
+  PAssert(activeDirection == Player, PLogicError);
+  return baseChannel != NULL && baseChannel->PlaySound(sound, wait);
 }
 
 
 PBoolean PSoundChannel::PlayFile(const PFilePath & file, PBoolean wait)
 {
-    return baseChannel != NULL && baseChannel->PlayFile(file, wait);
+  PAssert(activeDirection == Player, PLogicError);
+  return baseChannel != NULL && baseChannel->PlayFile(file, wait);
 }
 
 
 PBoolean PSoundChannel::HasPlayCompleted()
 {
-    return baseChannel == NULL || baseChannel->HasPlayCompleted();
+  PAssert(activeDirection == Player, PLogicError);
+  return baseChannel != NULL && baseChannel->HasPlayCompleted();
 }
 
 
 PBoolean PSoundChannel::WaitForPlayCompletion() 
 {
-    return baseChannel == NULL || baseChannel->WaitForPlayCompletion();
+  PAssert(activeDirection == Player, PLogicError);
+  return baseChannel != NULL && baseChannel->WaitForPlayCompletion();
 }
 
 
 PBoolean PSoundChannel::Read(void * buf, PINDEX len)
 {
-    return baseChannel != NULL && baseChannel->Read(buf, len);
+  PAssert(activeDirection == Recorder, PLogicError);
+
+  if (len == 0)
+    return IsOpen();
+
+  return baseChannel != NULL && baseChannel->Read(buf, len);
 }
 
 
 PINDEX PSoundChannel::GetLastReadCount() const
 {
-    return baseChannel == NULL ? lastReadCount : baseChannel->GetLastReadCount();
+  return baseChannel != NULL ? baseChannel->GetLastReadCount() : PChannel::GetLastReadCount();
 }
 
 
 PBoolean PSoundChannel::RecordSound(PSound & sound)
 {
-    return baseChannel != NULL && baseChannel->RecordSound(sound);
+  PAssert(activeDirection == Recorder, PLogicError);
+  return baseChannel != NULL && baseChannel->RecordSound(sound);
 }
 
 
 PBoolean PSoundChannel::RecordFile(const PFilePath & file)
 {
-    return baseChannel != NULL && baseChannel->RecordFile(file);
+  PAssert(activeDirection == Recorder, PLogicError);
+  return baseChannel != NULL && baseChannel->RecordFile(file);
 }
 
 
 PBoolean PSoundChannel::StartRecording()
 {
-    return baseChannel == NULL || baseChannel->StartRecording();
+  PAssert(activeDirection == Recorder, PLogicError);
+  return baseChannel != NULL && baseChannel->StartRecording();
 }
 
 
 PBoolean PSoundChannel::IsRecordBufferFull() 
 {
-    return baseChannel == NULL || baseChannel->IsRecordBufferFull();
+  PAssert(activeDirection == Recorder, PLogicError);
+  return baseChannel != NULL && baseChannel->IsRecordBufferFull();
 }
 
 
 PBoolean PSoundChannel::AreAllRecordBuffersFull() 
 {
-    return baseChannel == NULL || baseChannel->AreAllRecordBuffersFull();
+  PAssert(activeDirection == Recorder, PLogicError);
+  return baseChannel != NULL && baseChannel->AreAllRecordBuffersFull();
 }
 
 
 PBoolean PSoundChannel::WaitForRecordBufferFull() 
 {
-    return baseChannel == NULL || baseChannel->WaitForRecordBufferFull();
+  PAssert(activeDirection == Recorder, PLogicError);
+  return baseChannel != NULL && baseChannel->WaitForRecordBufferFull();
 }
 
 
 PBoolean PSoundChannel::WaitForAllRecordBuffersFull() 
 {
-    return baseChannel == NULL || baseChannel->WaitForAllRecordBuffersFull();
+  PAssert(activeDirection == Recorder, PLogicError);
+  return baseChannel != NULL && baseChannel->WaitForAllRecordBuffersFull();
 }
 
 
