@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23291 $
+ * $Revision: 25079 $
  * $Author: rjongbloed $
- * $Date: 2009-08-28 01:07:44 -0500 (Fri, 28 Aug 2009) $
+ * $Date: 2011-01-20 12:23:32 -0600 (Thu, 20 Jan 2011) $
  */
 
 #include <ptlib.h>
@@ -48,43 +48,11 @@
 #include <ptlib/wm/mmsystemx.h>
 #endif
 
-namespace PWLibStupidLinkerHacks {
-   int loadWindowsMultimediaStuff;
-};
-     
+
 class PSound;
 
 PCREATE_SOUND_PLUGIN(WindowsMultimedia, PSoundChannelWin32);
 
-class PMultiMediaFile
-{
-  public:
-    PMultiMediaFile();
-    ~PMultiMediaFile();
-
-    PBoolean CreateWaveFile(const PFilePath & filename,
-                        const PWaveFormat & waveFormat,
-                        DWORD dataSize);
-    PBoolean OpenWaveFile(const PFilePath & filename,
-                      PWaveFormat & waveFormat,
-                      DWORD & dataSize);
-
-    PBoolean Open(const PFilePath & filename, DWORD dwOpenFlags, LPMMIOINFO lpmmioinfo = NULL);
-    PBoolean Close(UINT wFlags = 0);
-    PBoolean Ascend(MMCKINFO & ckinfo, UINT wFlags = 0);
-    PBoolean Descend(UINT wFlags, MMCKINFO & ckinfo, LPMMCKINFO lpckParent = NULL);
-    PBoolean Read(void * data, PINDEX len);
-    PBoolean CreateChunk(MMCKINFO & ckinfo, UINT wFlags = 0);
-    PBoolean Write(const void * data, PINDEX len);
-
-    DWORD GetLastError() const { return dwLastError; }
-
-  protected:
-    HMMIO hmmio;
-    DWORD dwLastError;
-};
-
-
 #define new PNEW
 
 
@@ -935,6 +903,9 @@ PBoolean PSoundChannelWin32::Close()
 
 PBoolean PSoundChannelWin32::SetBuffers(PINDEX size, PINDEX count)
 {
+  if (count == buffers.GetSize() && size == buffers[0].GetSize())
+	  return true;
+
   Abort();
 
   PAssert(size > 0 && count > 0, PInvalidParameter);
@@ -1366,12 +1337,12 @@ PBoolean PSoundChannelWin32::Abort()
 
     if (hWaveOut != NULL || hWaveIn != NULL) {
       for (PINDEX i = 0; i < buffers.GetSize(); i++) {
-        while (buffers[i].Release() == WAVERR_STILLPLAYING) {
+        do {
           if (hWaveOut != NULL)
             waveOutReset(hWaveOut);
           if (hWaveIn != NULL)
             waveInReset(hWaveIn);
-        }
+        } while (buffers[i].Release() == WAVERR_STILLPLAYING);
       }
     }
 
