@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22378 $
+ * $Revision: 26448 $
  * $Author: rjongbloed $
- * $Date: 2009-04-06 22:20:13 -0500 (Mon, 06 Apr 2009) $
+ * $Date: 2011-09-19 22:01:53 -0500 (Mon, 19 Sep 2011) $
  */
 
 #include <ptlib.h>
@@ -171,76 +171,36 @@ static bool TrayIconRegistry(PServiceProcess * svc, TrayIconRegistryCommand cmd)
 
 
 ///////////////////////////////////////////////////////////////////////////////
-// PSystemLog
+// Logging
 
-void PSystemLog::Output(Level level, const char * msg)
+void PServiceProcess::LogToWindow::Output(PSystemLog::Level level, const char * msg)
 {
   if (msg == NULL || *msg == '\0')
     return;
 
   PServiceProcess & process = PServiceProcess::Current();
-  if (level > process.GetLogLevel())
-    return;
-
-  DWORD err = ::GetLastError();
-
   if (process.isWin95 || process.controlWindow != NULL) {
     static HANDLE mutex = CreateMutex(NULL, false, NULL);
     WaitForSingleObject(mutex, INFINITE);
 
-    ostream * out;
-    if (process.systemLogFileName == WindowLogOutput || process.systemLogFileName == DebuggerLogOutput)
-      out = new PStringStream;
-    else
-      out = new ofstream(process.systemLogFileName, ios::app);
+    PStringStream str;
+    OutputToStream(str, level, msg);
+    process.DebugOutput(str);
 
-    PTime now;
-    *out << now.AsString("yyyy/MM/dd hh:mm:ss.uuu\t");
-    PThread * thread = PThread::Current();
-    PString threadName;
-    if (thread == NULL)
-      threadName.sprintf("ThreadID" PTHREAD_ID_FMT, GetCurrentThreadId());
-    else
-      threadName = thread->GetThreadName();
-    if (threadName.GetLength() <= 23)
-      *out << setw(23) << threadName;
-    else
-      *out << threadName.Left(10) << "..." << threadName.Right(10);
-
-    *out << '\t';
-    if (level < 0)
-      *out << "Message";
-    else {
-      static const char * const levelName[4] = {
-        "Fatal error",
-        "Error",
-        "Warning",
-        "Info"
-      };
-      if (level < PARRAYSIZE(levelName))
-        *out << levelName[level];
-      else
-        *out << "Debug" << (level-Info);
-    }
-
-    *out << '\t' << msg;
-    if (level < Info && err != 0)
-      *out << " - error = " << err << endl;
-    else if (msg[0] == '\0' || msg[strlen(msg)-1] != '\n')
-      *out << endl;
-
-    if (process.systemLogFileName == WindowLogOutput)
-      process.DebugOutput(*(PStringStream*)out);
-    else if (process.systemLogFileName == DebuggerLogOutput)
-      OutputDebugStringA(*(PStringStream*)out);
-
-    delete out;
     ReleaseMutex(mutex);
     SetLastError(0);
   }
-  else {
+}
+
+
+class PSystemLogToEvent : public PSystemLogTarget
+{
+  virtual void Output(PSystemLog::Level level, const char * msg)
+  {
+    DWORD err = GetLastError();
+
     // Use event logging to log the error.
-    HANDLE hEventSource = RegisterEventSource(NULL, process.GetName());
+    HANDLE hEventSource = RegisterEventSource(NULL, PProcess::Current().GetName());
     if (hEventSource == NULL)
       return;
 
@@ -260,7 +220,7 @@ void PSystemLog::Output(Level level, const char * msg)
     }
 
     char errbuf[25];
-    if (level > StdError && level < Info && err != 0)
+    if (level > PSystemLog::StdError && level < PSystemLog::Info && err != 0)
       ::sprintf(errbuf, "Error code = %d", err);
     else
       errbuf[0] = '\0';
@@ -269,17 +229,16 @@ void PSystemLog::Output(Level level, const char * msg)
     strings[0] = thrdbuf;
     strings[1] = msg;
     strings[2] = errbuf;
-    strings[3] = level != Fatal ? "" : " Program aborted.";
+    strings[3] = level != PSystemLog::Fatal ? "" : " Program aborted.";
 
-    static const WORD levelType[Info+1] = {
+    static const WORD levelType[PSystemLog::Info+1] = {
       EVENTLOG_INFORMATION_TYPE,
       EVENTLOG_ERROR_TYPE,
       EVENTLOG_ERROR_TYPE,
       EVENTLOG_WARNING_TYPE
     };
     ReportEvent(hEventSource, // handle of event source
-                (WORD)(level < Info ? levelType[level+1]
-                                    : EVENTLOG_INFORMATION_TYPE), // event type
+      (WORD)(level < PSystemLog::Info ? levelType[level+1] : EVENTLOG_INFORMATION_TYPE), // event type
                 (WORD)(level+1),      // event category
                 0x1000,               // event ID
                 NULL,                 // current user's SID
@@ -289,62 +248,7 @@ void PSystemLog::Output(Level level, const char * msg)
                 NULL);                // no raw data
     DeregisterEventSource(hEventSource);
   }
-}
-
-
-streambuf::int_type PSystemLog::Buffer::overflow(int c)
-{
-  if (pptr() >= epptr()) {
-    PMEMORY_IGNORE_ALLOCATIONS_FOR_SCOPE;
-
-    int ppos = pptr() - pbase();
-    char * newptr = string.GetPointer(string.GetSize() + 10);
-    setp(newptr, newptr + string.GetSize() - 1);
-    pbump(ppos);
-  }
-  if (c != EOF) {
-    *pptr() = (char)c;
-    pbump(1);
-  }
-
-  return 0;
-}
-
-
-streambuf::int_type PSystemLog::Buffer::underflow()
-{
-  return EOF;
-}
-
-
-int PSystemLog::Buffer::sync()
-{
-  Level logLevel = log->logLevel;
-
-#if PTRACING
-  if (log->width() != 0 &&(PTrace::GetOptions()&PTrace::SystemLogStream) != 0) {
-    // Trace system sets the ios stream width as the last thing it does before
-    // doing a flush, which gets us here. SO now we can get a PTRACE looking
-    // exactly like a PSYSTEMLOG of appropriate level.
-    unsigned traceLevel = log->width() -1 + PSystemLog::Warning;
-    log->width(0);
-    if (traceLevel >= PSystemLog::NumLogLevels)
-      traceLevel = PSystemLog::NumLogLevels-1;
-    logLevel = (Level)traceLevel;
-  }
-#endif
-
-  PSystemLog::Output(logLevel, string);
-
-  PMEMORY_IGNORE_ALLOCATIONS_FOR_SCOPE;
-
-  string.SetSize(10);
-  char * base = string.GetPointer();
-  *base = '\0';
-  setp(base, base + string.GetSize() - 1);
- 
-  return 0;
-}
+};
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -352,11 +256,9 @@ int PSystemLog::Buffer::sync()
 
 PServiceProcess::PServiceProcess(const char * manuf, const char * name,
                            WORD major, WORD minor, CodeStatus stat, WORD build)
-  : PProcess(manuf, name, major, minor, stat, build),
-    systemLogFileName(GetFile().GetDirectory() + GetName() + " Log.TXT")
+  : PProcess(manuf, name, major, minor, stat, build)
 {
   controlWindow = debugWindow = NULL;
-  currentLogLevel = PSystemLog::Warning;
 }
 
 
@@ -376,7 +278,7 @@ const char * PServiceProcess::GetServiceDependencies() const
 
 PBoolean PServiceProcess::IsServiceProcess() const
 {
-  return true;
+  return !debugMode;
 }
 
 
@@ -429,7 +331,7 @@ int PServiceProcess::InternalMain(void * arg)
   debugHidden = arguments.GetCount() > 0 && strcasecmp(arguments[0], "DebugHidden") == 0;
   if (debugHidden)
     debugMode=TRUE;
-  currentLogLevel = debugMode ? PSystemLog::Info : PSystemLog::Warning;
+  SetLogLevel(debugMode ? PSystemLog::Info : PSystemLog::Warning);
 
   if (!debugMode && arguments.GetCount() > 0) {
     for (PINDEX a = 0; a < arguments.GetCount(); a++)
@@ -459,11 +361,13 @@ int PServiceProcess::InternalMain(void * arg)
     };
     dispatchTable[0].lpServiceName = (char *)(const char *)GetName();
 
+    PSystemLog::SetTarget(new PSystemLogToEvent());
+
     if (StartServiceCtrlDispatcher(dispatchTable))
       return GetTerminationValue();
 
     if (GetLastError() != ERROR_FAILED_SERVICE_CONTROLLER_CONNECT)
-      PSystemLog::Output(PSystemLog::Fatal, "StartServiceCtrlDispatcher failed.");
+      PSYSTEMLOG(Fatal, "StartServiceCtrlDispatcher failed.");
     ProcessCommand(ServiceCommandNames[SvcCmdDefault]);
     return 1;
   }
@@ -531,18 +435,20 @@ int PServiceProcess::InternalMain(void * arg)
     }
   } while (msg.message != WM_QUIT);
 
-  if (controlWindow != NULL)
+  if (controlWindow != NULL && controlWindow != (HWND)-1)
     DestroyWindow(controlWindow);
 
   // Set thread ID for process to this thread
-  activeThreadMutex.Wait();
-  activeThreads.SetAt(threadId, NULL);
+  m_activeThreadMutex.Wait();
+  m_activeThreads.erase(threadId);
   threadId = GetCurrentThreadId();
   threadHandle = GetCurrentThread();
-  activeThreads.SetAt(threadId, this);
-  activeThreadMutex.Signal();
+  m_activeThreads[threadId] = this;
+  m_activeThreadMutex.Signal();
   OnStop();
 
+  controlWindow = NULL; // This stops the logging
+
   return GetTerminationValue();
 }
 
@@ -686,13 +592,17 @@ PBoolean PServiceProcess::CreateControlWindow(PBoolean createDebugWindow)
     SendMessage(debugWindow, EM_SETTABSTOPS, PARRAYSIZE(TabStops), (LPARAM)(LPDWORD)TabStops);
 
 #if P_CONFIG_FILE
-    systemLogFileName = cfg.GetString(SystemLogFileNameKey);
+    PString systemLogFileName = cfg.GetString(SystemLogFileNameKey);
 #endif // P_CONFIG_FILE
-    if (systemLogFileName.IsEmpty())
-      systemLogFileName = WindowLogOutput;
-    if (systemLogFileName != WindowLogOutput) {
-      if (systemLogFileName != DebuggerLogOutput)
+    if (systemLogFileName.IsEmpty() || systemLogFileName == WindowLogOutput)
+      PSystemLog::SetTarget(new LogToWindow());
+    else {
+      if (systemLogFileName == DebuggerLogOutput)
+        PSystemLog::SetTarget(new PSystemLogToDebug());
+      else {
         PFile::Remove(systemLogFileName);
+        PSystemLog::SetTarget(new PSystemLogToFile(systemLogFileName));
+      }
       DebugOutput("Sending all system log output to \"" + systemLogFileName + "\".\n");
     }
   }
@@ -738,7 +648,7 @@ LPARAM PServiceProcess::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lPara
         nid.Delete(); // This removes the systray icon
       }
 
-      controlWindow = debugWindow = NULL;
+      controlWindow = debugWindow = (HWND)-1;
 
       PostQuitMessage(0);
       break;
@@ -865,14 +775,15 @@ LPARAM PServiceProcess::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lPara
             fileDlgInfo.Flags = OFN_ENABLEHOOK|OFN_HIDEREADONLY|OFN_NOVALIDATE|OFN_EXPLORER|OFN_CREATEPROMPT;
             fileDlgInfo.lCustData = (DWORD)this;
             if (GetSaveFileName(&fileDlgInfo)) {
-              if (systemLogFileName != fileBuffer) {
-                systemLogFileName = fileBuffer;
-                PFile::Remove(systemLogFileName);
+              PFilePath newLogFile = fileBuffer;
+              if (!PIsDescendant(&PSystemLog::GetTarget(), PSystemLogToFile) ||
+                    ((PSystemLogToFile &)PSystemLog::GetTarget()).GetFilePath() != newLogFile) {
+                PSystemLog::SetTarget(new PSystemLogToFile(newLogFile));
 #if P_CONFIG_FILE
                 PConfig cfg(ServiceSimulationSectionName);
-                cfg.SetString(SystemLogFileNameKey, systemLogFileName);
+                cfg.SetString(SystemLogFileNameKey, newLogFile);
 #endif // P_CONFIG_FILE
-                DebugOutput("Sending all system log output to \"" + systemLogFileName + "\".\n");
+                DebugOutput("Sending all system log output to \"" + newLogFile + "\".\n");
                 PError << "Logging started for \"" << GetName() << "\" version " << GetVersion(true) << endl;
               }
             }
@@ -880,25 +791,23 @@ LPARAM PServiceProcess::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lPara
           break;
 
         case WindowOutputMenuID :
-          if (systemLogFileName != WindowLogOutput) {
+          if (!PIsDescendant(&PSystemLog::GetTarget(), LogToWindow)) {
             PError << "Logging stopped." << endl;
-            DebugOutput("System log output to \"" + systemLogFileName + "\" stopped.\n");
-            systemLogFileName = WindowLogOutput;
+            PSystemLog::SetTarget(new LogToWindow());
 #if P_CONFIG_FILE
             PConfig cfg(ServiceSimulationSectionName);
-            cfg.SetString(SystemLogFileNameKey, systemLogFileName);
+            cfg.SetString(SystemLogFileNameKey, WindowLogOutput);
 #endif // P_CONFIG_FILE
           }
           break;
 
         case DebuggerOutputMenuID :
-          if (systemLogFileName != DebuggerLogOutput) {
+          if (!PIsDescendant(&PSystemLog::GetTarget(), PSystemLogToDebug)) {
             PError << "Logging stopped." << endl;
-            DebugOutput("System log output to \"" + systemLogFileName + "\" stopped.\n");
-            systemLogFileName = DebuggerLogOutput;
+            PSystemLog::SetTarget(new PSystemLogToDebug());
 #if P_CONFIG_FILE
             PConfig cfg(ServiceSimulationSectionName);
-            cfg.SetString(SystemLogFileNameKey, systemLogFileName);
+            cfg.SetString(SystemLogFileNameKey, DebuggerLogOutput);
 #endif // P_CONFIG_FILE
           }
           break;
@@ -1109,11 +1018,11 @@ void PServiceProcess::StaticThreadEntry(void * arg)
 
 void PServiceProcess::ThreadEntry()
 {
-  activeThreadMutex.Wait();
+  m_activeThreadMutex.Wait();
   threadId = GetCurrentThreadId();
   threadHandle = GetCurrentThread();
-  activeThreads.SetAt(threadId, this);
-  activeThreadMutex.Signal();
+  m_activeThreads[threadId] = this;
+  m_activeThreadMutex.Signal();
 
   SetTerminationValue(1);
   bool ok = OnStart();
@@ -1199,7 +1108,7 @@ PBoolean PServiceProcess::ReportStatus(DWORD dwCurrentState,
     return true;
 
   // If an error occurs, stop the service.
-  PSystemLog::Output(PSystemLog::Error, "SetServiceStatus failed");
+  PSYSTEMLOG(Error, "SetServiceStatus failed");
   return false;
 }
 
@@ -1651,7 +1560,6 @@ bool PServiceProcess::ProcessCommand(const char * cmd)
         nid.uCallbackMessage = UWM_SYSTRAY; // message sent to nid.hWnd
         nid.Add();    // This adds the icon
         debugWindow = (HWND)-1;
-        systemLogFileName = DebuggerLogOutput;
         return true;
       }
       return false;
@@ -1669,10 +1577,15 @@ bool PServiceProcess::ProcessCommand(const char * cmd)
 
     case SvcCmdVersion : // Version command
       ::SetLastError(0);
-      PError << GetName() << " Version " << GetVersion(true)
-             << " by " << GetManufacturer()
-             << " on " << GetOSClass()   << ' ' << GetOSName()
-             << " ("   << GetOSVersion() << '-' << GetOSHardware() << ')' << endl;
+      {
+        PStringStream msg;
+        msg << GetName() << " Version " << GetVersion(true)
+            << " by " << GetManufacturer()
+            << " on " << GetOSClass()   << ' ' + GetOSName()
+            << " ("   << GetOSVersion() << '-' + GetOSHardware() << ')';
+        MessageBox(NULL, msg, GetName(), MB_TASKMODAL);
+        PError << msg << endl;
+      }
       return true;
 
     case SvcCmdDefault : // run app with no params.
