@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24285 $
+ * $Revision: 26872 $
  * $Author: rjongbloed $
- * $Date: 2010-04-26 01:18:11 -0500 (Mon, 26 Apr 2010) $
+ * $Date: 2012-01-19 22:19:40 -0600 (Thu, 19 Jan 2012) $
  */
 
 #include <ptlib.h>
@@ -47,10 +47,12 @@
   #pragma comment(lib, "mpr.lib")
 #endif
 
+#include <ptlib/msos/ptlib/ipsock.h>
+
 #if defined(P_WIN_COM) 
   #include <objbase.h>
   #ifdef _MSC_VER
-    #pragma comment(lib, _OLE_LIB)
+    #pragma comment(lib, "ole32.lib")
   #endif
 #endif
 
@@ -61,7 +63,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 // PTime
 
-PTime::PTime()
+void PTime::SetCurrentTime()
 {
   FILETIME timestamp;
 
@@ -495,14 +497,14 @@ PBoolean PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clust
 ///////////////////////////////////////////////////////////////////////////////
 // PFilePath
 
-static PString IllegalFilenameCharacters =
+static char const IllegalFilenameCharacters[] =
   "\\/:*?\"<>|"
   "\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\0x10"
   "\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f";
 
 PBoolean PFilePath::IsValid(char c)
 {
-  return IllegalFilenameCharacters.Find(c) == P_MAX_INDEX;
+  return strchr(IllegalFilenameCharacters, c) == NULL;
 }
 
 
@@ -593,7 +595,6 @@ PBoolean PChannel::ConvertOSError(int status, Errors & lastError, int & osError)
     switch (osError) {
       case ERROR_INVALID_HANDLE :
       case WSAEBADF :
-      case WSAENOTSOCK :
         osError = EBADF;
         break;
       case ERROR_INVALID_PARAMETER :
@@ -610,15 +611,63 @@ PBoolean PChannel::ConvertOSError(int status, Errors & lastError, int & osError)
       case WSAEINTR :
         osError = EINTR;
         break;
+      case WSAEINPROGRESS :
+        osError = EINPROGRESS;
+        break;
+      case WSAENOTSOCK :
+        osError = ENOTSOCK;
+        break;
+      case WSAEOPNOTSUPP :
+        osError = EOPNOTSUPP;
+        break;
+      case WSAEAFNOSUPPORT :
+        osError = EAFNOSUPPORT;
+        break;
+      case WSAEADDRINUSE :
+        osError = EADDRINUSE;
+        break;
+      case WSAEADDRNOTAVAIL :
+        osError = EADDRNOTAVAIL;
+        break;
+      case WSAENETDOWN :
+        osError = ENETDOWN;
+        break;
+      case WSAENETUNREACH :
+        osError = ENETUNREACH;
+        break;
+      case WSAENETRESET :
+        osError = ENETRESET;
+        break;
+      case WSAECONNABORTED :
+        osError = ECONNABORTED;
+        break;
+      case WSAECONNRESET :
+        osError = ECONNRESET;
+        break;
+      case WSAENOBUFS :
+        osError = ENOBUFS;
+        break;
+      case WSAEISCONN :
+        osError = EISCONN;
+        break;
+      case WSAENOTCONN :
+        osError = ENOTCONN;
+        break;
+      case WSAECONNREFUSED :
+        osError = ECONNREFUSED;
+        break;
+      case WSAEHOSTUNREACH :
+        osError = EHOSTUNREACH;
+        break;
       case WSAEMSGSIZE :
-        osError |= PWIN32ErrorFlag;
-        lastError = BufferTooSmall;
-        return PFalse;
+        osError = EMSGSIZE;
+        break;
       case WSAEWOULDBLOCK :
+        osError = EWOULDBLOCK;
+        break;
       case WSAETIMEDOUT :
-        osError |= PWIN32ErrorFlag;
-        lastError = Timeout;
-        return PFalse;
+        osError = ETIMEDOUT;
+        break;
       default :
         osError |= PWIN32ErrorFlag;
     }
@@ -650,8 +699,13 @@ PBoolean PChannel::ConvertOSError(int status, Errors & lastError, int & osError)
       lastError = NotOpen;
       break;
     case EAGAIN :
+    case ETIMEDOUT :
+    case EWOULDBLOCK :
       lastError = Timeout;
       break;
+    case EMSGSIZE :
+      lastError = BufferTooSmall;
+      break;
     case EINTR :
       lastError = Interrupted;
       break;
@@ -842,6 +896,9 @@ bool PThread::GetTimes(Times & times)
 
 void PThread::CleanUp()
 {
+  if (threadHandle == NULL)
+    return;
+
   PProcess & process = PProcess::Current();
   process.m_activeThreadMutex.Wait();
   process.m_activeThreads.erase(threadId);
@@ -850,8 +907,8 @@ void PThread::CleanUp()
   if (!IsTerminated())
     Terminate();
 
-  if (threadHandle != NULL)
-    CloseHandle(threadHandle);
+  CloseHandle(threadHandle);
+  threadHandle = NULL;
 }
 
 
@@ -877,7 +934,8 @@ void PThread::Restart()
 
 void PThread::Terminate()
 {
-  PAssert(originalStackSize > 0, PLogicError);
+  if (!PAssert(!m_isProcess, "Cannot terminate the process!"))
+    return;
 
   if (Current() == this)
     ExitThread(0);
@@ -904,7 +962,7 @@ void PThread::WaitForTermination() const
 PBoolean PThread::WaitForTermination(const PTimeInterval & maxWait) const
 {
   if ((this == PThread::Current()) || threadHandle == NULL) {
-    PTRACE(3, "PWLib\tWaitForTermination short circuited");
+    PTRACE(3, "PTLib\tWaitForTermination short circuited");
     return PTrue;
   }
 
@@ -948,7 +1006,7 @@ void PThread::Resume()
 
 PBoolean PThread::IsSuspended() const
 {
-  if (threadHandle == PThread::Current()->GetHandle())
+  if (this == PThread::Current())
     return false;
 
   SuspendThread(threadHandle);
@@ -959,23 +1017,24 @@ PBoolean PThread::IsSuspended() const
 void PThread::SetAutoDelete(AutoDeleteFlag deletion)
 {
   PAssert(deletion != AutoDeleteThread || this != &PProcess::Current(), PLogicError);
+  bool newAutoDelete = (deletion == AutoDeleteThread);
+  if (autoDelete == newAutoDelete)
+    return;
+
+  autoDelete = newAutoDelete;
 
   PProcess & process = PProcess::Current();
 
-  if (autoDelete && deletion != AutoDeleteThread) {
-    process.deleteThreadMutex.Wait();
+  process.deleteThreadMutex.Wait();
+  if (autoDelete)
+    process.autoDeleteThreads.Append(this);
+  else {
     process.autoDeleteThreads.DisallowDeleteObjects();
     process.autoDeleteThreads.Remove(this);
     process.autoDeleteThreads.AllowDeleteObjects();
-    process.deleteThreadMutex.Signal();
-  }
-  else if (!autoDelete && deletion == AutoDeleteThread) {
-    process.deleteThreadMutex.Wait();
-    process.autoDeleteThreads.Append(this);
-    process.deleteThreadMutex.Signal();
   }
+  process.deleteThreadMutex.Signal();
 
-  autoDelete = deletion == AutoDeleteThread;
 }
 
 
@@ -1025,7 +1084,7 @@ void PThread::Yield()
 // PProcess::TimerThread
 
 PProcess::HouseKeepingThread::HouseKeepingThread()
-  : PThread(1000, NoAutoDeleteThread, NormalPriority, "PTLib Housekeeper")
+  : PThread(1000, NoAutoDeleteThread, HighestPriority, "PTLib Housekeeper")
   , m_running(true)
 {
   Resume();
@@ -1064,7 +1123,7 @@ void PProcess::HouseKeepingThread::Main()
 #ifndef _WIN32_WCE
         DWORD dwFlags;
         if (GetHandleInformation(handles[numHandles], &dwFlags) == 0) {
-          PTRACE(2, "PWLib\tRefused to put invalid handle into wait list");
+          PTRACE(2, "PTLib\tRefused to put invalid handle into wait list");
         }
         else
 #endif
@@ -1133,37 +1192,49 @@ bool PProcess::SignalTimerChange()
 
 PProcess::~PProcess()
 {
+  PTRACE(4, "PTLib\tStarting process destruction.");
+
   // do whatever needs to shutdown
   PreShutdown();
 
   Sleep(100);  // Give threads time to die a natural death
 
   // Get rid of the house keeper (majordomocide)
+  PTRACE(4, "PTLib\tTerminating housekeeper thread.");
   delete houseKeeper;
   houseKeeper = NULL;
 
   // OK, if there are any left we get really insistent...
   m_activeThreadMutex.Wait();
+  PTRACE(4, "PTLib\tTerminating " << m_activeThreads.size()-1 << " remaining threads.");
   for (ThreadMap::iterator it = m_activeThreads.begin(); it != m_activeThreads.end(); ++it) {
     PThread & thread = *it->second;
-    if (this != &thread && !thread.IsTerminated())
-      TerminateThread(thread.GetHandle(), 1);  // With extreme prejudice
+    if (this != &thread && !thread.IsTerminated()) {
+      PTRACE(3, "PTLib\tTerminating thread " << thread);
+      thread.Terminate();  // With extreme prejudice
+    }
   }
+  PTRACE(4, "PTLib\tTerminated all threads.");
   m_activeThreadMutex.Signal();
 
   deleteThreadMutex.Wait();
+  PTRACE(4, "PTLib\tDestroying " << autoDeleteThreads.GetSize() << " remaining auto-delete threads.");
   autoDeleteThreads.RemoveAll();
   deleteThreadMutex.Signal();
 
-  PostShutdown();
-
-#if PTRACING
-  PTrace::Cleanup();
-#endif
-
 #if _DEBUG
   WaitOnExitConsoleWindow();
 #endif
+
+  PTRACE(4, "PTLib\tCompleted process destruction.");
+
+  // Can't do any more tracing after this ...
+#if PTRACING
+  PTrace::Cleanup();
+  PTrace::SetStream(NULL);
+#endif
+
+  PostShutdown();
 }
 
 
@@ -1260,6 +1331,26 @@ PString PProcess::GetOSVersion()
 }
 
 
+bool PProcess::IsOSVersion(unsigned major, unsigned minor, unsigned build)
+{
+  OSVERSIONINFO info;
+  info.dwOSVersionInfoSize = sizeof(info);
+  GetVersionEx(&info);
+
+  if (info.dwMajorVersion < major)
+    return false;
+  if (info.dwMajorVersion > major)
+    return true;
+
+  if (info.dwMinorVersion < minor)
+    return false;
+  if (info.dwMinorVersion > minor)
+    return true;
+
+  return info.dwBuildNumber >= build;
+}
+
+
 PDirectory PProcess::GetOSConfigDir()
 {
 #ifdef _WIN32_WCE
