@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23344 $
+ * $Revision: 24524 $
  * $Author: rjongbloed $
- * $Date: 2009-09-05 01:12:00 -0500 (Sat, 05 Sep 2009) $
+ * $Date: 2010-06-21 20:05:24 -0500 (Mon, 21 Jun 2010) $
  */
 
 #define _OSUTIL_CXX
@@ -163,6 +163,38 @@ PString PProcess::GetOSVersion()
 #endif
 }
 
+bool PProcess::IsOSVersion(unsigned major, unsigned minor, unsigned build)
+{
+#if defined(HAS_UNAME)
+  struct utsname info;
+  uname(&info);
+  unsigned maj, min, bld;
+  sscanf(
+#ifdef P_SOLARIS
+         info.version
+#else
+         info.release
+#endif
+         , "%u.%u.%u", &maj, &min, &bld);
+  if (maj < major)
+    return false;
+  if (maj > major)
+    return true;
+
+  if (min < minor)
+    return false;
+  if (min > minor)
+    return true;
+
+  return bld >= build;
+#elif defined(P_VXWORKS)
+  return sysBspRev() >= major;
+#else
+  return true;
+#endif
+}
+
+
 PDirectory PProcess::GetOSConfigDir()
 {
 #ifdef P_VXWORKS
