@@ -26,9 +26,9 @@
  *
  * Contributor(s): Yuri Kiryanov, ykiryanov at users.sourceforge.net
  *
- * $Revision: 21219 $
+ * $Revision: 25645 $
  * $Author: rjongbloed $
- * $Date: 2008-10-02 03:52:00 -0500 (Thu, 02 Oct 2008) $
+ * $Date: 2011-05-01 19:19:49 -0500 (Sun, 01 May 2011) $
  */
 
 class PThread;
@@ -423,7 +423,7 @@ void PThread::PXAbortBlock(void) const
 PDECLARE_CLASS(PHouseKeepingThread, PThread)
   public:
     PHouseKeepingThread()
-      : PThread(1000, NoAutoDeleteThread, NormalPriority, "Housekeeper")
+      : PThread(1000, NoAutoDeleteThread, HighestPriority, "Housekeeper")
       { closing = PFalse; Resume(); }
 
     void Main();
