@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24753 $
+ * $Revision: 26573 $
  * $Author: rjongbloed $
- * $Date: 2010-09-27 21:11:47 -0500 (Mon, 27 Sep 2010) $
+ * $Date: 2011-10-10 05:54:20 -0500 (Mon, 10 Oct 2011) $
  */
 
 #include <ptlib/socket.h>
@@ -146,7 +146,7 @@ static int GetSchedParam(PThread::Priority priority, sched_param & param)
 PDECLARE_CLASS(PHouseKeepingThread, PThread)
   public:
     PHouseKeepingThread()
-      : PThread(1000, NoAutoDeleteThread, NormalPriority, "Housekeeper")
+      : PThread(1000, NoAutoDeleteThread, HighestPriority, "Housekeeper")
       { closing = PFalse; Resume(); }
 
     void Main();
@@ -496,7 +496,11 @@ void * PThread::PX_ThreadStart(void * arg)
   // make sure the cleanup routine is called when the thread exits
   //pthread_cleanup_push(&PThread::PX_ThreadEnd, arg);
 
+#if defined(P_LINUX)
+  PTRACE(5, "PTLib\tStarted thread " << thread << " (" << thread->PX_linuxId << ") " << thread->GetThreadName());
+#else
   PTRACE(5, "PTLib\tStarted thread " << thread << ' ' << thread->GetThreadName());
+#endif
 
   PProcess::Current().OnThreadStart(*thread);
 
@@ -580,7 +584,8 @@ void PThread::Restart()
   // unlock the thread list
   process.m_activeThreadMutex.Signal();
 
-  PTRACE_IF(4, newHighWaterMark > 0, "PTLib\tThread high water mark set: " << newHighWaterMark);
+  PTRACE_IF(newHighWaterMark%100 == 0 ? 2 : 4, newHighWaterMark > 0,
+            "PTLib\tThread high water mark set: " << newHighWaterMark);
 
 #ifdef P_MACOSX
   if (PX_priority == HighestPriority) {
@@ -963,9 +968,9 @@ PBoolean PThread::WaitForTermination(const PTimeInterval & maxWait) const
 
   PXAbortBlock();   // this assist in clean shutdowns on some systems
 
-  PTimeInterval startWaitTick = PTimer::Tick();
+  PSimpleTimer timeout(maxWait);
   while (!IsTerminated()) {
-    if ((PTimer::Tick() - startWaitTick) > maxWait)
+    if (timeout.HasExpired())
       return false;
 
     Sleep(10); // sleep for 10ms. This slows down the busy loop removing 100%
@@ -977,6 +982,8 @@ PBoolean PThread::WaitForTermination(const PTimeInterval & maxWait) const
 }
 
 
+#if defined(P_LINUX)
+
 static inline unsigned long long jiffies_to_msecs(const unsigned long j)
 {
   static long sysconf_HZ = sysconf(_SC_CLK_TCK);
@@ -984,54 +991,171 @@ static inline unsigned long long jiffies_to_msecs(const unsigned long j)
 }
 
 
-bool PThread::GetTimes(Times & times)
+static bool LinuxGetTimes(const PString & statFileName,
+#if PTRACING
+                          PString & error,
+#endif
+                          PThread::Times & times)
 {
-#if defined(P_LINUX)
-  PStringStream procname;
-  procname << "/proc/" << getpid() << "/task/" << PX_linuxId << "/stat";
-  PTextFile statfile(procname, PFile::ReadOnly);
+  /* From the man page on the "stat" file
+      Status information about the process. This is used by ps(1). It is defined in /usr/src/linux/fs/proc/array.c.
+      The fields, in order, with their proper scanf(3) format specifiers, are:
+         pid         %d   The process ID.
+         comm        %s   The filename of the executable, in parentheses. This is visible
+                          whether or not the executable is swapped out.
+         state       %c   One character from the string "RSDZTW" where R is running, S is
+                          sleeping in an interruptible wait, D is waiting in uninterruptible
+                          disk sleep, Z is zombie, T is traced or stopped (on a signal), and
+                          W is paging.
+         ppid        %d   The PID of the parent.
+         pgrp        %d   The process group ID of the process.
+         session     %d   The session ID of the process.
+         tty_nr      %d   The tty the process uses.
+         tpgid       %d   The process group ID of the process which currently owns the tty
+                          that the process is connected to.
+         flags       %lu  The kernel flags word of the process. For bit meanings, see the
+                          PF_* defines in <linux/sched.h>. Details depend on the kernel
+                          version.
+         minflt      %lu  The number of minor faults the process has made which have not
+                          required loading a memory page from disk.
+         cminflt     %lu  The number of minor faults that the process's waited-for children
+                          have made.
+         majflt      %lu  The number of major faults the process has made which have required
+                          loading a memory page from disk.
+         cmajflt     %lu  The number of major faults that the process's waited-for children
+                          have made.
+         utime       %lu  The number of jiffies that this process has been scheduled in user
+                          mode.
+         stime       %lu  The number of jiffies that this process has been scheduled in kernel
+                          mode.
+         cutime      %ld  The number of jiffies that this process's waited-for children have
+                          been scheduled in user mode. (See also times(2).)
+         cstime      %ld  The number of jiffies that this process's waited-for children have
+                          been scheduled in kernel mode.
+         priority    %ld  The standard nice value, plus fifteen. The value is never negative
+                          in the kernel.
+         nice        %ld  The nice value ranges from 19 (nicest) to -19 (not nice to others).
+         num_threads %ld  Number of threads.
+         itrealvalue %ld  The time in jiffies before the next SIGALRM is sent to the process
+                          due to an interval timer.
+         starttime   %lu  The time in jiffies the process started after system boot.
+         vsize       %lu  Virtual memory size in bytes.
+         rss         %ld  Resident Set Size: number of pages the process has in real memory,
+                          minus 3 for administrative purposes. This is just the pages which
+                          count towards text, data, or stack space. This does not include
+                          pages which have not been demand-loaded in, or which are swapped out.
+         rlim        %lu  Current limit in bytes on the rss of the process
+                          (usually 4294967295 on i386).
+         startcode   %lu  The address above which program text can run.
+         endcode     %lu  The address below which program text can run.
+         startstack  %lu  The address of the start of the stack.
+         kstkesp     %lu  The current value of esp (stack pointer), as found in the kernel
+                          stack page for the process.
+         kstkeip     %lu  The current EIP (instruction pointer).
+         signal      %lu  The bitmap of pending signals.
+         blocked     %lu  The bitmap of blocked signals.
+         sigignore   %lu  The bitmap of ignored signals.
+         sigcatch    %lu  The bitmap of caught signals.
+         wchan       %lu  This is the "channel" in which the process is waiting. It is the
+                          address of a system call, and can be looked up in a namelist if you
+                          need a textual name. (If you have an up-to-date /etc/psdatabase, then
+                          try ps -l to see the WCHAN field in action.)
+         nswap       %lu  Number of pages swapped (not maintained).
+         cnswap      %lu  Cumulative nswap for child processes (not maintained).
+         exit_signal %d   Signal to be sent to parent when we die.
+         processor   %d   CPU number last executed on.
+         rt_priority %lu  (since kernel 2.5.19) Real-time scheduling priority (see sched_setscheduler(2)).
+         policy      %lu  (since kernel 2.5.19) Scheduling policy (see sched_setscheduler(2)).
+         delayacct_blkio_ticks %llu (since Linux 2.6.18) Aggregated block I/O delays, measured in
+                          clock ticks (centiseconds).
+  */
+
+  PTextFile statfile(statFileName, PFile::ReadOnly);
   if (!statfile.IsOpen()) {
-    PTRACE(2, "PTLib\tCould not find thread stat file " << procname);
+    PTRACE_PARAM(error = "Could not find thread stat file");
     return false;
   }
 
-  statfile.ignore(10000, ')'); // Skip pid & filename of executable
-
-  char state;
-  unsigned long dummy, utime, stime;
-  statfile >> state
-           >> dummy // ppid
-           >> dummy // pgrp
-           >> dummy // session
-           >> dummy // tty_nr
-           >> dummy // tpgid
-           >> dummy // flags
-           >> dummy // minflt
-           >> dummy // cminflt
-           >> dummy // majflt
-           >> dummy // cmajflt
-           >> utime
-           >> stime;
+  char line[1000];
+  statfile.getline(line, sizeof(line));
   if (!statfile.good()) {
-    PTRACE(2, "PTLib\tCould not parse thread stat file " << procname);
+    PTRACE_PARAM(error = "Could not read thread stat file");
+    return false;
+  }
+
+  int pid;
+  char comm[100];
+  char state;
+  int ppid, pgrp, session, tty_nr, tpgid;
+  unsigned long flags, minflt, cminflt, majflt, cmajflt, utime, stime;
+  long cutime, cstime, priority, nice, num_threads, itrealvalue;
+  unsigned long starttime, vsize;
+  long rss;
+  unsigned long rlim, startcode, endcode, startstack, kstkesp, kstkeip, signal, blocked, sigignore, sigcatch, wchan, nswap, cnswap;
+  int exit_signal, processor;
+  unsigned long rt_priority, policy;
+  unsigned long long delayacct_blkio_ticks;
+
+  // 17698 (maxmcu) R 1 17033 8586 34833 17467 4202560 7
+  // 0 0 0 0 0 0 0 -100 0 16
+  // 0 55172504 258756608 6741 4294967295 134512640 137352760 3217892976 8185700 15991824
+  // 0 0 4 201349635 0 0 0 -1 7 99
+  // 2 0
+
+  int count = sscanf(line,
+         "%d%s %c%d%d%d%d%d%lu%lu"
+         "%lu%lu%lu%lu%lu%ld%ld%ld%ld%ld"
+         "%ld%lu%lu%ld%lu%lu%lu%lu%lu%lu"
+         "%lu%lu%lu%lu%lu%lu%lu%d%d%lu"
+         "%lu%llu",
+         &pid, comm, &state, &ppid, &pgrp, &session, &tty_nr, &tpgid, &flags, &minflt,
+         &cminflt, &majflt, &cmajflt, &utime, &stime, &cutime, &cstime, &priority, &nice, &num_threads,
+         &itrealvalue, &starttime, &vsize, &rss, &rlim, &startcode, &endcode, &startstack, &kstkesp, &kstkeip,
+         &signal, &blocked, &sigignore, &sigcatch, &wchan, &nswap, &cnswap, &exit_signal, &processor, &rt_priority,
+         &policy, &delayacct_blkio_ticks);
+  if (count != 42) {
+    PTRACE_PARAM(error = psprintf("Not enough values (%d)\n%s", count, line));
     return false;
   }
 
   times.m_kernel = jiffies_to_msecs(stime);
   times.m_user = jiffies_to_msecs(utime);
-
-  if (PX_endTick != 0)
-    times.m_real = PX_endTick - PX_startTick;
-  else
-    times.m_real = PTimer::Tick() - PX_startTick;
-
   return true;
-#else
-  return false;
-#endif
 }
 
 
+bool PThread::GetTimes(Times & times)
+{
+  PStringStream statFileName;
+  statFileName << "/proc/" << getpid() << "/task/" << PX_linuxId << "/stat";
+
+  PTRACE_PARAM(PString error);
+
+  for (int retry = 0; retry < 3; ++retry) {
+    if (LinuxGetTimes(statFileName,
+#if PTRACING
+                      error,
+#endif
+                      times)) {
+      if (PX_endTick != 0)
+        times.m_real = PX_endTick - PX_startTick;
+      else
+        times.m_real = PTimer::Tick() - PX_startTick;
+      return true;
+    }
+  }
+
+  PTRACE(2, "PTLib\tError reading " << statFileName << ", " << error);
+  return false;
+}
+#else
+bool PThread::GetTimes(Times & times)
+{
+  return false;
+}
+#endif
+
+
 int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
 {
   PTRACE(7, "PTLib\tPThread::PXBlockOnIO(" << handle << ',' << type << ')');
@@ -1211,7 +1335,7 @@ sem_t * PSemaphore::CreateSem(unsigned initialValue)
   
   PAssertPTHREAD(pthread_mutex_unlock, (&semCreationMutex));
   
-  PAssert(((int)sem != (int)SEM_FAILED), "Couldn't create named semaphore");
+  PAssert(sem != SEM_FAILED, "Couldn't create named semaphore");
   return sem;
 }
 #endif
@@ -1263,12 +1387,13 @@ PBoolean PSemaphore::Wait(const PTimeInterval & waitTime)
   absTime.tv_sec  = finishTime.GetTimeInSeconds();
   absTime.tv_nsec = finishTime.GetMicrosecond() * 1000;
 
-  if (sem_timedwait(&semId, &absTime) == 0) {
-    return PTrue;
-  }
-  else {
-    return PFalse;
-  }
+  do {
+    if (sem_timedwait(&semId, &absTime) == 0)
+      return true;
+  } while (errno == EINTR);
+
+  PAssert(errno == ETIMEDOUT, strerror(errno));
+  return false;
 
 #else
   // loop until timeout, or semaphore becomes available
