@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23434 $
- * $Author: rjongbloed $
- * $Date: 2009-09-14 02:35:24 -0500 (Mon, 14 Sep 2009) $
+ * $Revision: 26169 $
+ * $Author: csoutheren $
+ * $Date: 2011-07-11 03:25:38 -0500 (Mon, 11 Jul 2011) $
  */
 
 #pragma implementation "dynalink.h"
@@ -295,14 +295,13 @@ static PMutex & GetDLLMutex()
 #else
 
 PDynaLink::PDynaLink()
+  : dllHandle(NULL)
 {
-  dllHandle = NULL;
 }
 
 PDynaLink::PDynaLink(const PString & _name)
-  : name(_name)
+  : dllHandle(NULL)
 {
-  dllHandle = NULL;
   Open(_name);
 }
 
@@ -313,30 +312,40 @@ PDynaLink::~PDynaLink()
 
 PString PDynaLink::GetExtension()
 {
+#ifdef P_MACOSX
+  return PString(".dylib");
+#else
   return PString(".so");
+#endif
 }
 
 PBoolean PDynaLink::Open(const PString & _name)
 {
+  PWaitAndSignal m(GetDLLMutex());
+
+  m_lastError.MakeEmpty();
+
   Close();
 
+  if (_name.IsEmpty())
+    return false;
+
+  PTRACE(4, "UDLL\topening " << _name);
+
   name = _name;
 
   {
-    PWaitAndSignal m(GetDLLMutex());
-
-    const char *err; //= dlerror();
-
 #if defined(P_OPENBSD)
     dllHandle = dlopen((char *)(const char *)name, RTLD_NOW);
 #else
     dllHandle = dlopen((const char *)name, RTLD_NOW);
 #endif
 
-    if (dllHandle == NULL) {
-      err = dlerror();
-      PTRACE_IF(1, err != NULL, "DLL\tError loading DLL - " << err);
-    }
+    if (dllHandle != NULL)
+      return true;
+
+    m_lastError = dlerror();
+    PTRACE(1, "DLL\tError loading DLL: " << m_lastError);
   }
 
   return IsLoaded();
@@ -344,10 +353,16 @@ PBoolean PDynaLink::Open(const PString & _name)
 
 void PDynaLink::Close()
 {
-  if (dllHandle != NULL) {
-    dlclose(dllHandle);
-    dllHandle = NULL;
-  }
+  PWaitAndSignal m(GetDLLMutex());
+
+  if (dllHandle == NULL)
+    return;
+
+  PTRACE(4, "UDLL\tClosing " << name);
+
+
+  dlclose(dllHandle);
+  dllHandle = NULL;
   name.MakeEmpty();
 }
 
@@ -358,6 +373,8 @@ PBoolean PDynaLink::IsLoaded() const
 
 PString PDynaLink::GetName(PBoolean full) const
 {
+  PWaitAndSignal m(GetDLLMutex());
+
   if (!IsLoaded())
     return "";
 
@@ -365,12 +382,14 @@ PString PDynaLink::GetName(PBoolean full) const
     return name;
 
   PString str = name;
-  PINDEX pos = str.FindLast('/');
-  if (pos != P_MAX_INDEX)
-    str = str.Mid(pos+1);
-  pos = str.FindLast(".so");
-  if (pos != P_MAX_INDEX)
-    str = str.Left(pos);
+  if (!full) {
+    PINDEX pos = str.FindLast('/');
+    if (pos != P_MAX_INDEX)
+      str = str.Mid(pos+1);
+    pos = str.FindLast(".so");
+    if (pos != P_MAX_INDEX)
+      str = str.Left(pos);
+  }
 
   return str;
 }
@@ -383,20 +402,24 @@ PBoolean PDynaLink::GetFunction(PINDEX, Function &)
 
 PBoolean PDynaLink::GetFunction(const PString & fn, Function & func)
 {
+  PWaitAndSignal m(GetDLLMutex());
+
+  m_lastError.MakeEmpty();
+
   if (dllHandle == NULL)
     return PFalse;
 
 #if defined(P_OPENBSD)
-  void * p = dlsym(dllHandle, (char *)(const char *)fn);
+  func = (Function)dlsym(dllHandle, (char *)(const char *)fn);
 #else
-  void * p = dlsym(dllHandle, (const char *)fn);
+  func = (Function)dlsym(dllHandle, (const char *)fn);
 #endif
 
-  if (p == NULL)
-    return PFalse;
+  if (func != NULL)
+    return true;
 
-  func = (Function)p;
-  return PTrue;
+  m_lastError = dlerror();
+  return false;
 }
 
 #endif
